function job=mr_cfg_fMRIfilter
%temporal filter
%created 15/09-2015
%Kristoffer H. Madsen (kristofferm@drcmr.dk)

RT          = cfg_entry;
RT.tag      = 'RT';
RT.name     = 'Repitition time (TR)';
RT.help     = {'Only relevant for DCT highpass filter and physiological noise regressors.'};
RT.strtype  ='r';
RT.num      = [1 1];
RT.val      = {[2.]};

dcthp       = cfg_entry;
dcthp.tag   = 'dcthp';
dcthp.name  = 'Highpass filter cut-off in Hz';
dcthp.help  = {'Discrete cosine transform based highpass filter cut-off frequency, select 0 to disable. Default: 1/128 Hz'};
dcthp.strtype='r';
dcthp.num   = [1 1];
dcthp.val   = {[1./128]};

dctlp       = cfg_entry;
dctlp.tag   = 'dctlp';
dctlp.name  = 'Lowpass filter cut-off in Hz';
dctlp.help  = {'Discrete cosine transform based lowpass filter cut-off frequency, select Inf to disable.'};
dctlp.strtype='r';
dctlp.num   = [1 1];
dctlp.val   = {Inf};

input       = cfg_files;
input.name  = 'Scans';
input.tag   = 'input';
input.filter= 'image';
input.ufilter= '.*';
input.num =  [1 inf];
input.help = {'Select volumes if they are not resliced to same orientation this will be done using 4th order b-spline interpolation.'};

pdetrend       = cfg_entry;
pdetrend.tag   = 'pdetrend';
pdetrend.name  = 'Polynomial detrending order';
pdetrend.help  = {'Include polynominal drift nuisance regressors up to the defined order. 1 selects linear drift term, select 0 to disable.'};
pdetrend.strtype='w';
pdetrend.num   = [1 1];
pdetrend.val   = {[0]};

NVR_motion          = cfg_files;
NVR_motion.name='Realignment file';
NVR_motion.tag='NVR_motion';
NVR_motion.filter='rp_.*';
NVR_motion.num=[0 1];
NVR_motion.val      = {''};
NVR_motion.help     = {'Specify containing movement parameters (normally from SPM realignment procedure). Leave blank to omit modelling of residual movement effects.'};

NVR_motion_type     = cfg_menu;
NVR_motion_type.name='Realignment regressors';
NVR_motion_type.tag='NVR_motion_type';
NVR_motion_type.labels={'All - basic, squared, spin history, spin history squared (24)','Basic + spin history (12)','No spin history (12)','Basic'};
NVR_motion_type.values={1, 2, 3, 4};
NVR_motion_type.val ={1};

NVR_FD     = cfg_menu;
NVR_FD.name='Displacement';
NVR_FD.tag='NVR_FD';
NVR_FD.labels={'Framewise (Power)','Euclidean'};
NVR_FD.values={1,0};
NVR_FD.val  = {1};
NVR_FD.help = {'Calculation of displacements.'};

savefilter     = cfg_menu;
savefilter.name='Save filter in mat file';
savefilter.tag='savefilter';
savefilter.labels={'Yes','No'};
savefilter.values={1,0};
savefilter.val  = {1};
savefilter.help = {'Save filter in .mat file - contains the filter (X) and residual forming matrix (R). File will be named as the 4D output but with _reg.mat extension instead of .nii.'};

despike    = cfg_menu;
despike.name='Wavelet despiking';
despike.tag='despike';
despike.labels={'Yes','No'};
despike.values={1,0};
despike.val  = {1};
despike.help = {'Use wavelet despiking.'};

despikereg    = cfg_menu;
despikereg.name='Spike percentage in filter';
despikereg.tag='despikereg';
despikereg.labels={'Yes','No'};
despikereg.values={1,0};
despikereg.val  = {0};
despikereg.help = {'Include despike percentage in temporal filter.'};

despikefirst    = cfg_menu;
despikefirst.name='Despike before filtering';
despikefirst.tag='despikefirst';
despikefirst.labels={'Yes','No'};
despikefirst.values={1,0};
despikefirst.val  = {1};
despikefirst.help = {'Do despiking as first step (before filtering).'};

limRAM     = cfg_entry;
limRAM.name='Max. memory to use during despiking';
limRAM.tag='limRAM';
limRAM.num=[1 1];
limRAM.strtype='r';
limRAM.val  = {8};
limRAM.help = {'Allow this amount of memory to be used during wavelet despiking.'};

wtype    = cfg_menu;
wtype.name='Wavelet type';
wtype.tag='wtype';
wtype.labels={'d4','d6','d8','d10','d12','d14','d16','d18','d20','la8',...
    'la10','la12','la14','la16','la18','la20','bl14','bl18','bl20',...
    'c6','c12','c18','c24','haar'};
wtype.values={'d4','d6','d8','d10','d12','d14','d16','d18','d20','la8',...
    'la10','la12','la14','la16','la18','la20','bl14','bl18','bl20',...
    'c6','c12','c18','c24','haar'};
wtype.val  = {'d4'};
wtype.help = {'Do despiking as first step (before filtering).'};

wthres= cfg_entry;
wthres.name='Wavelet coefficient threshold.';
wthres.tag='wthres';
wthres.num=[1 1];
wthres.strtype='r';
wthres.val  = {10};
wthres.help = {'Threshold for maximal and minimal wavelet coefficients.'};

boundary    = cfg_menu;
boundary.name='Boundaries type';
boundary.tag='boundary';
boundary.labels={'periodic','circular','reflection'};
boundary.values={'periodic','circular','reflection'};
boundary.val  = {'reflection'};
boundary.help = {''};

chtype    = cfg_menu;
chtype.name='Rules for min max chains';
chtype.tag='chtype';
chtype.labels={'conservative','moderate','harsh'};
chtype.values={'conservative','moderate','harsh'};
chtype.val  = {'moderate'};
chtype.help = {''};

nscale    = cfg_menu;
nscale.name='No. scales';
nscale.tag='nscale';
nscale.labels={'conservative','liberal','extreme'};
nscale.values={'conservative','liberal','extreme'};
nscale.val  = {'liberal'};
nscale.help = {''};

compress    = cfg_menu;
compress.name='Compress non-brain';
compress.tag='compress';
compress.labels={'yes','no'};
compress.values={1,0};
compress.val  = {1};
compress.help = {''};

%                            series to despike.
%               Opref     -- Output prefix for despiked NIfTI files and
%                            Spike Percentage (if specified).
%
%               Additional Input Options:
%               (These must be specific as MATLAB string-value pairs.)
%
%               wavelet   -- Wavelet to use for wavelet transform. Input
%                            must be a string containing one of the 
%                            following:
%                            'd4','d6','d8','d10','d12','d14','d16','d18',
%                            'd20','la8','la10','la12','la14','la16',
%                            'la18','la20','bl14','bl18','bl20','c6','c12',
%                            'c18','c24','haar'. [Default='d4'].
%               threshold -- Threshold for maximal and minimal wavelet
%                            coefficients. [Default=10].
%               boundary  -- Boundary condition to apply. Input must be a
%                            string containing one of the following: 
%                            'circular','reflection'.
%                            [Default='reflection'].
%               chsearch  -- Rules for identifying maxima and minima 
%                            chains. Input must be a string containing one 
%                            of the following: 'conservative', 'moderate',
%                            'harsh'. [Default='moderate'].
%               nscale    -- Method for computing number of scales. Input
%                            must be a string containing one of the
%                            following: 'conservative','liberal','extreme'.
%                            [Default: 'liberal'].
%               compress  -- Binary flag indicating whether to compress out
%                            non-brain regions, 1, or not, 0, from input 
%                            NIfTI file before Wavelet Despiking. This 
%                            saves RAM and reduces runtime. [Default=1].
%               sp        -- Binary flag indicating whether to output the
%                            Spike Percentage for the dataset.
%                            [Default=1].
%               LimitRAM  -- Specify an upper bound of RAM usage (in Giga-
%                            bytes). Default is to use all available RAM.
%               verbose   -- Binary flag indicating whether to display
%                            incremental output from the algorithm, 1, or
%                            not, 0.



NVR_FD_rad     = cfg_entry;
NVR_FD_rad.name='Head radius in mm';
NVR_FD_rad.tag='NVR_FD_rad';
NVR_FD_rad.num=[1 1];
NVR_FD_rad.strtype='r';
NVR_FD_rad.val  = {50};
NVR_FD_rad.help = {'Used in the calculation of framewise displacements only.'};

NVR_motion_threshold_rel  = cfg_entry;
NVR_motion_threshold_rel.name='Relative movement threshold';
NVR_motion_threshold_rel.tag='NVR_motion_threshold_rel';
NVR_motion_threshold_rel.num=[1 1];
NVR_motion_threshold_rel.strtype='r';
NVR_motion_threshold_rel.val  = {inf};
NVR_motion_threshold_rel.help = {['If movement relative to the previous volume larger than this threshold is detected during realignment a predefined number of subsequent volumes (see next option) can be excluded.']...
                             ['The threshold is defined on the estimated Euclidean displacement between volumes (only translational parameters included).']};

NVR_motion_threshold_abs  = cfg_entry;
NVR_motion_threshold_abs.name='Absolute movement threshold';
NVR_motion_threshold_abs.tag='NVR_motion_threshold_abs';
NVR_motion_threshold_abs.strtype='r';
NVR_motion_threshold_abs.num=[1 1];
NVR_motion_threshold_abs.val  = {inf};
NVR_motion_threshold_abs.help = {['If absolute movement larger than this threshold is detected (relative to first volume) during realignment a predefined number of subsequent volumes (see next option) can be excluded. ']...
                             ['The threshold is defined on the estimated Euclidean displacement between volumes (only translational parameters included).']};
                         
NVR_motion_exclude  = cfg_entry;
NVR_motion_exclude.name='Exclude volumes before/after movement';
NVR_motion_exclude.tag='NVR_motion_exclude';
NVR_motion_exclude.strtype='i';
NVR_motion_exclude.num=[1 inf];
NVR_motion_exclude.val  = {[0 1]};
NVR_motion_exclude.help = {['Relative indices of volumes to exclude before/after movement larger than the defined threshold has been detected ' ...
    '(zero is the volume that exceeded the threshold, -1 the volume before and 1 the volume after).']};

DVARS = cfg_entry;
DVARS.name='Motion threshold (DVARS)';
DVARS.tag='DVARS';
DVARS.strtype='r';
DVARS.num=[1 1];
DVARS.val  = {inf};
DVARS.help = {['If DVARS value larger than this is detected  a predefined number of subsequent volumes (see next option) can be excluded. '] ...
              ['DVARS is the absolute change in mean signal value from volume to volume (only voxels that are present in all volumes will be included in the mean).']};

NVR_motion_svd  = cfg_entry;
NVR_motion_svd.name='Singular vectors to include';
NVR_motion_svd.tag='NVR_motion_svd';
NVR_motion_svd.strtype='i';
NVR_motion_svd.num=[1 1];
NVR_motion_svd.val  = {[0]};
NVR_motion_svd.help = {['If this is set to a positive number, singular value decomposition will be done on the motion parameters' ...
    'retaining only this number of singular vectors (ordered by singular values). ' ...
    'Note that there is an arbitary weighting between translation and rotation, '...
    'the implementation currently scales the total variance of translation, rotation, higher orders and spin history effects to one prior to the SVD (i.e. in blocks of 3).']};

NVR_mask_file          = cfg_files;
NVR_mask_file.name='NVR mask image';
NVR_mask_file.tag='NVR_mask_file';
NVR_mask_file.filter='.*';
NVR_mask_file.num=[0 1];
NVR_mask_file.filter   = 'image';
NVR_mask_file.help     = {['Create nuisance regressors from mask, this option will construct' ...
    'nuisance regressors based on the signal within a mask.']};

NVR_mask_th = cfg_entry;
NVR_mask_th.name='NVR Mask threshold';
NVR_mask_th.tag='NVR_mask_th';
NVR_mask_th.strtype='r';
NVR_mask_th.num=[1 1];
NVR_mask_th.val = {0};
NVR_mask_th.help = {'Mask threshold used to create NVR regressors - regions with mask values larger than this will be used.'};

NVR_mask_svd = cfg_entry;
NVR_mask_svd.name='No. singular vectors to use for NVR modelling';
NVR_mask_svd.tag='NVR_mask_svd';
NVR_mask_svd.strtype='i';
NVR_mask_svd.num=[1 1];
NVR_mask_svd.val = {0};
NVR_mask_svd.help = {'Number of singular vector to use as NVR regressors. Zero (default) will use omit svd and use the mean.'};

NVR_mask_erode = cfg_entry;
NVR_mask_erode.name='Erode mask radius';
NVR_mask_erode.tag='NVR_mask_erode';
NVR_mask_erode.strtype='r';
NVR_mask_erode.num=[1 1];
NVR_mask_erode.val = {0};
NVR_mask_erode.help = {'If set to positive number will 3D erode the mask with a spherical filter with this radius in voxels (1st image). Require the image processing toolbox.'};

NVR_deriv      = cfg_menu;
NVR_deriv.name='Also add derivative';
NVR_deriv.tag='NVR_deriv';
NVR_deriv.labels={'no','yes'};
NVR_deriv.values={0,1};
NVR_deriv.val  = {[0]};
NVR_deriv.help = {['Include derivative.']};

NVR_mask         = cfg_branch;
NVR_mask.tag     = 'NVR_mask';
NVR_mask.name    = 'Data in mask';
NVR_mask.val     = {NVR_mask_file,NVR_mask_th,NVR_mask_svd,NVR_mask_erode,NVR_deriv};
NVR_mask.help    = {''};

NVR_point_xyz          = cfg_entry;
NVR_point_xyz.name='[x,y,z] point';
NVR_point_xyz.tag='NVR_point_xyz';
NVR_point_xyz.strtype='r';
NVR_point_xyz.num=[1 3];
NVR_point_xyz.help     = {['Create nuisance regressors from point, this option will construct' ...
    'nuisance regressors based on the signal in a region around a point.']};

NVR_point_r = cfg_entry;
NVR_point_r.name='Radius around point';
NVR_point_r.tag='NVR_point_r';
NVR_point_r.strtype='r';
NVR_point_r.num=[1 1];
NVR_point_r.val = {0};
NVR_point_r.help = {'Radius of sphere around point at which data will be used.'};

NVR_point         = cfg_branch;
NVR_point.tag     = 'NVR_point';
NVR_point.name    = 'Data around point';
NVR_point.val     = {NVR_point_xyz,NVR_point_r,NVR_mask_svd,NVR_deriv};
NVR_point.help    = {''};

exvol      = cfg_entry;
exvol.name='Extra volumes to exclude';
exvol.tag='exvol';
exvol.strtype='e';
exvol.num=[0 inf];
exvol.val  = {[]};
exvol.help = {['Additional volumes to be excluded.']};

%%Physiological file - Philips
physfile          = cfg_files;
physfile.name='Physiological log file - Philips';
physfile.tag='physfile';
physfile .filter='.*';
physfile .num=[0 1];
physfile.val={''};
physfile .help     = {['Logfile from Philips scanner - SCANPHYSLOG. If Siemens scanner is used, leave empty.']};

% Siemens
physsiemens          = cfg_files;
physsiemens.name     = 'Physiological log file';
physsiemens.tag      = 'physsiemens';
physsiemens.filter   = {'.resp','.puls','.ecg'}; 
physsiemens.num      = [0 1];
physsiemens.val      = {''};
physsiemens.help     = {['Either the .puls, .resp or .ecg file from the Siemens scanner. The filenames for the files should be the same, i.e. {logname}.puls, {logname}.resp and {logname}.ecg. If Philips scanner is used, leave empty. ']};

% pulseon               = cfg_menu;
% pulseon.name          = 'Use pulse-file';
% pulseon.tag           = 'pulseon';
% pulseon.labels        = {'Yes','No'};
% pulseon.values        = {'ON','OFF'};
% pulseon.val           = {'OFF'} ;
% pulseon.help          = {'Select Yes if a .puls file is used from the Siemens scanner. If Philips scanner is used, leave at No. '};
% 
% respon               = cfg_menu;
% respon.name          = 'Use respiration-file';
% respon.tag           = 'respon';
% respon.labels        = {'Yes','No'};
% respon.values        = {'ON','OFF'};
% respon.val           = {'OFF'} ;
% respon.help          = {'Select Yes if a .resp file is used from the Siemens scanner. If Philips scanner is used, leave at No'};
% 
% ecgon                 = cfg_menu;
% ecgon.name            = 'Use ECG-file';
% ecgon.tag             = 'ecgon';
% ecgon.labels          = {'Yes','No'};
% ecgon.values          = {'ON','OFF'};
% ecgon.val             = {'OFF'} ;
% ecgon.help            = {'Select Yes if a .ecg file is used from the Siemens scanner. If Philips scanner is used, leave at No'};

lp_pulseon            = cfg_menu;
lp_pulseon.name       = 'Filter the pulse time series';
lp_pulseon.tag        = 'lp_pulseon';
lp_pulseon.labels     = {'Yes','No'};
lp_pulseon.values     = {'ON','OFF'};
lp_pulseon.val        = {'OFF'} ;
lp_pulseon.help       = {'Select Yes if the .puls file from the Siemens scanner should be filtered. If Philips scanner is used, leave at No'};

lp_respon            = cfg_menu;
lp_respon.name       = 'Filter the respiration time series';
lp_respon.tag        = 'lp_respon';
lp_respon.labels     = {'Yes','No'};
lp_respon.values     = {'ON','OFF'};
lp_respon.val        = {'OFF'} ;
lp_respon.help       = {'Select Yes if the .resp file from the Siemens scanner should be filtered. If Philips scanner is used, leave at No'};

lp_ecgon            = cfg_menu;
lp_ecgon.name       = 'Filter the ECG time series';
lp_ecgon.tag        = 'lp_ecgon';
lp_ecgon.labels     = {'Yes','No'};
lp_ecgon.values     = {'ON','OFF'};
lp_ecgon.val        = {'OFF'} ;
lp_ecgon.help       = {'Select Yes if the .ecg file from the Siemens scanner should be filtered. If Philips scanner is used, leave at No'};

ndummies            = cfg_entry;
ndummies.name       = 'No. dummies';
ndummies.tag        = 'ndummies';
ndummies.strtype    = 'w';
ndummies.num        = [1 1];
ndummies.val        = {0};
ndummies.help       = {'No. of dummy volumes before the first scan. If Philips scanner is used, leave at 0.'};

br                  = cfg_branch;
br.tag              = 'br';
br.name             = 'Siemens';
br.val              = {physsiemens,lp_pulseon,lp_respon,lp_ecgon,ndummies};

block               = cfg_repeat;
block.tag           = 'block';
block.name          = 'Siemens data';
block.help          = {'If data from Siemens scanner is used'};
block.values        = {br};
block.num           = [0 1];

rvton                 = cfg_menu;
rvton.name            = 'Add RVT-regressor ';
rvton.tag             = 'rvton';
rvton.labels          = {'Yes','No'};
rvton.values          = {'ON','OFF'};
rvton.val             = {'OFF'} ;
rvton.help            = {'Select Yes if the respiration volume per time regressor (RVT-regressor) should be added.'};

corder = cfg_entry;
corder.name='Cardiac expansion order';
corder.tag='corder';
corder.strtype='w';
corder.num=[1 inf];
corder.val = {2};
corder.help = {''};

rorder = cfg_entry;
rorder.name='Respiratory expansion order';
rorder.tag='rorder';
rorder.strtype='w';
rorder.num=[1 inf];
rorder.val = {2};
rorder.help = {''};

ecgorder = cfg_entry;
ecgorder.name='ECG expansion order';
ecgorder.tag='ecgorder';
ecgorder.strtype='w';
ecgorder.num=[1 inf];
ecgorder.val = {2};
ecgorder.help = {''};

crorder = cfg_entry;
crorder.name='Cardiac by respiratory interaction order';
crorder.tag='crorder';
crorder.strtype='w';
crorder.num=[1 inf];
crorder.val = {0};
crorder.help = {''};

nslices = cfg_entry;
nslices.name='No. slices';
nslices.tag='nslices';
nslices.strtype='w';
nslices.num=[1 1];
nslices.val = {0};
nslices.help = {'No. of slices in original data, leave at 0 to detect from input file, note that detection from resampled/normalised files will not work correctly due to potentially changed no. slices.'};

sonset = cfg_entry;
sonset.name='Physio slice onset';
sonset.tag='sonset';
sonset.strtype='w';
sonset.num=[1 1];
sonset.val = {0};
sonset.help = {'Offset regressors to this slice (temporal slice order), if left at zero this will be the middle of the volume (in time).'};

dregs         = cfg_repeat;
dregs.tag     = 'NVR_mask_rep';
dregs.name    = 'Nuisance regressors from data';
dregs.help    = {'Construct nuisance regressors from data'};
dregs.values  = {NVR_mask,NVR_point};
dregs.num     = [0 Inf];

missingregs      = cfg_menu;
missingregs.name='Handling of missing data';
missingregs.tag='missing';
missingregs.labels={'Drop regressors','Add random regressor','Crash with error'};
missingregs.values={1,2,3};
missingregs.val  = {3};
missingregs.help = {'Choose how missing data in nuisance variables should be handled.'};

prefix = cfg_entry;
prefix.name='Output prefix';
prefix.tag='prefix';
prefix.strtype='s';
prefix.num=[1 inf];
prefix.val = {'f'};
prefix.help = {'Prefix to add to output files.'};

addmean  = cfg_menu;
addmean.name='Add mean to residuals';
addmean.tag='addmean';
addmean.labels={'No','Yes'};
addmean.values={0,1};
addmean.val  = {[1]} ;
addmean.help= {'Add mean to residuals after fitting otherwise timeseries will be zero mean after fitting.'};

savedata  = cfg_menu;
savedata.name='Write out residuals';
savedata.tag='savedata';
savedata.labels={'No','Yes'};
savedata.values={0,1};
savedata.val  = {[1]} ;
savedata.help= {'Write out the resulting residuals after fitting otherwise only file with regressors will be saved.'};

%extra regressors
NVR_extra = cfg_entry;
NVR_extra.name='Extra nuisance regressors';
NVR_extra.tag='NVR_extra';
NVR_extra.strtype='r';
NVR_extra.num=[inf inf];
NVR_extra.val   = {[]};
NVR_extra.help  = {'Enter extra nuisance regressors.'};

job=cfg_exbranch;
job.tag='fMRIfilter';
job.name='Temporal filter';
scriptdir=fullfile(spm('dir'),'toolbox','tempfilter');
if isdir(fullfile(scriptdir,'..','PhysIO_r671'))
    job.val={input,RT,dcthp,pdetrend,dctlp,NVR_motion,NVR_motion_type,NVR_FD,NVR_FD_rad,NVR_motion_threshold_rel,NVR_motion_threshold_abs,NVR_motion_exclude,NVR_motion_svd,DVARS,dregs,...
        physfile,block,rvton,corder,rorder,ecgorder,crorder,nslices,sonset,NVR_extra,exvol,addmean,missingregs,prefix,savefilter,savedata};    
else
    job.val={input,RT,dcthp,pdetrend,dctlp,NVR_motion,NVR_motion_type,NVR_FD,NVR_FD_rad,NVR_motion_threshold_rel,NVR_motion_threshold_abs,NVR_motion_exclude,NVR_motion_svd,DVARS,dregs,NVR_extra,exvol,addmean,missingregs,prefix,savefilter,savedata};
end
if isdir(fullfile(scriptdir,'..','BrainWavelet_1.1'))
    job.val=[job.val ,{despike,despikefirst,despikereg,wtype,limRAM,wthres,chtype,boundary,nscale,compress}];
end

job.help = {'Filter fMRI data with linear temporal filter.'};
job.prog=@run_filter;
job.vout=@vout_filter;
end

function sts=check_ori(V)
matx = reshape(cat(3,V.mat),[16,numel(V)]);
sts=any(any(abs(diff(matx,1,2))>1e-4));
end

function job=run_filter(job)
scriptdir=fullfile(spm('dir'),'toolbox','tempfilter');
physdir=fullfile(scriptdir,'..','PhysIO_r671');
sdir=pwd;
rorder=4;
fprintf('Reading volumes... ')
Vin=spm_vol(strvcat(job.input));
sts=check_ori(Vin);
T=length(Vin);
if sts
    fprintf('Reslicing... ')
    [Y1, XYZ]=spm_read_vols(Vin(1));
    Y=zeros(size(Y1,1),size(Y1,2),size(Y1,3),T);
    Y(:,:,:,1)=Y1;
    clear Y1
    XYZ=[XYZ;ones(1,size(XYZ,2))];

    for i=2:length(Vin)
        sts1=check_ori(Vin([1 i]));
        if sts1
            coords=Vin(i).mat\XYZ;
            vol=spm_sample_vol(Vin(i),coords(1,:),coords(2,:),coords(3,:),rorder);
            vol=reshape(vol,Vin(1).dim);
        else
            vol=spm_read_vols(Vin(i));
        end
        Y(:,:,:,i)=vol;
    end
    clear vol
    clear coords
else
    Y=spm_read_vols(Vin);
end
sY=size(Y(:,:,:,1));
Y=reshape(Y,[prod(sY) T])';
fprintf('done\n')
[outdir,outname,ext]=fileparts(Vin(1).fname);

if isfield(job,'despike') && job.despike
wdOpts=struct('wavelet',job.wtype,'threshold',job.wthres,'boundary',job.boundary,...
    'chsearch',job.chtype,'nscale',job.nscale,'compress',job.compress,...
    'LimitRAM',job.limRAM,'verbose',1);
end

if isfield(job,'despikefirst') && job.despikefirst
    [Y,noise,SP]=runWaveletDespike(Y,wdOpts);
    clear noise
end

%% NVR section
% NVR
%-------------------------------------------------------------
rnames=[{'mean'}];
X=ones(T,1);
if job.pdetrend>0
    X0lin=linspace(0,1,T)';
    X0poly=ones(size(X0lin));
    names=[];
    for ii=1:job.pdetrend
        X0ptemp=X0lin.^ii;
        X0ptemp=X0ptemp/std(X0ptemp);
        X0ptemp=X0ptemp-X0poly/(X0poly'*X0poly)*X0poly'*X0ptemp;
        X0ptemp=X0ptemp/std(X0ptemp);
        X0poly=[X0poly X0ptemp];
        rnames=[rnames {sprintf('NVR_poly %d',ii)}];
    end
    X    = [X X0poly(:,2:end)];
    clear X0poly X0ptemp X0lin names
end
if job.dcthp>0 | (isfield(job,'dctlp')&&isfinite(job.dctlp))
     if not(isfield(job,'dctlp')) || not(isfinite(job.dctlp))
        Ktemp.RT=job.RT;
        Ktemp.HParam = 1./job.dcthp;
        Ktemp.row=1:T;
        Ktemp=spm_filter(Ktemp);
        X0=Ktemp.X0;
        X=[X X0];
        for ii=1:size(X0,2)
            rnames=[rnames {sprintf('NVR_hp %d',ii)}];
        end
        clear Ktemp
     else
        X0=spm_dctmtx(T,T);
        kmin       = fix(2*(T*job.RT)*job.dcthp + 1);
        kmax       = fix(2*(T*job.RT)*job.dctlp);
        if kmin>0
            X=[X X0(:,1:kmin)];
            for ii=1:kmin
                rnames=[rnames {sprintf('NVR_hp %d',ii)}];
            end
        end
        if kmax<T
            X=[X X0(:,kmax:T)];
            for ii=1:(T-kmax+1)
                rnames=[rnames {sprintf('NVR_lp %d',ii)}];
            end
        end
     end
 end
 motion_exclude=false(T,1);
 exclude_DVARS=false(T,1);
 exclude_rel=false(T,1);
 exclude_abs=false(T,1);
 if ~isempty(job.DVARS) && isfinite(job.DVARS) && job.DVARS>=0
    fprintf('Calculating DVARS ...')
    vars=mean(Y(:,all(isfinite(Y),1)),2);
    dvars   = (sqrt([diff(vars).^2])./abs([vars(1:(end-1))]))*100;
    exdvars = find(dvars>(job.DVARS));
    for ii=1:length(exdvars)
        for iii=1:length(job.NVR_motion_exclude)
            if (exdvars(ii)+1+job.NVR_motion_exclude(iii))>0 && (exdvars(ii)+1+job.NVR_motion_exclude(iii))<=T
                motion_exclude(exdvars(ii)+1+job.NVR_motion_exclude(iii))=1;
                exclude_DVARS(exdvars(ii)+1+job.NVR_motion_exclude(iii))=1;
            end
        end
    end
    dvars=[0;dvars];
end

    if iscell(job.NVR_motion) && ~isempty(job.NVR_motion{1})
        m = load(job.NVR_motion{1});
        
        if ~isempty(job.NVR_motion_exclude)
            if job.NVR_FD
                %framewise displacement (Powers et al.)
                FD=[[zeros(1,3);diff(m(:,1:3))] [zeros(1,3);diff(m(:,4:6))*job.NVR_FD_rad]];
                FD=sum(abs(FD),2);
                FD_abs=[m(:,1:3) m(:,4:6)*job.NVR_FD_rad];
                FD_abs=sum(abs(FD_abs),2);
                exmotion_r=find(FD>job.NVR_motion_threshold_rel)-1;
                exmotion_a=find(FD_abs>job.NVR_motion_threshold_abs)-1;
                
            else
                %relative motion
                exmotion_r =           find(sqrt(sum(diff(m(:,1:3)).^2,2))      > job.NVR_motion_threshold_rel);
                %absolute motion
                exmotion_a = find(sqrt(sum((m(:,1:3)-repmat(m(1,1:3),size(m,1),1)).^2,2)) > job.NVR_motion_threshold_abs)-1;    
            end
            exmotion=[exmotion_r;exmotion_a];
            relmotion=sqrt(sum(diff(m(:,1:3)).^2,2));
            absmotion=sqrt(sum((m(:,1:3)-repmat(m(1,1:3),size(m,1),1)).^2,2));
            for ii=1:length(exmotion_r)
                for iii=1:length(job.NVR_motion_exclude)
                    if (exmotion_r(ii)+1+job.NVR_motion_exclude(iii))>0 && (exmotion_r(ii)+1+job.NVR_motion_exclude(iii))<=T
                        motion_exclude(exmotion_r(ii)+1+job.NVR_motion_exclude(iii))=1;
                        exclude_rel(exmotion_r(ii)+1+job.NVR_motion_exclude(iii))=1;
                    end
                end
            end
            for ii=1:length(exmotion_a)
                for iii=1:length(job.NVR_motion_exclude)
                    if (exmotion_a(ii)+1+job.NVR_motion_exclude(iii))>0 && (exmotion_a(ii)+1+job.NVR_motion_exclude(iii))<=T
                        motion_exclude(exmotion_a(ii)+1+job.NVR_motion_exclude(iii))=1;
                        exclude_abs(exmotion_a(ii)+1+job.NVR_motion_exclude(iii))=1;
                    end
                end
            end
            
            ii=find(motion_exclude);
            %for iii=1:length(ii)
            %    fprintf('Excluding volume %d due to movement.\n',ii(iii));
            %end
            fprintf('Excluding %d volumes due to movement.\n',length(ii))
            clear ii iii
        end 

        switch job.NVR_motion_type
        case 1 % All parameters
            mh=[0 0 0 0 0 0;m(1:end-1,:)];
            reg=[m m.^2 mh mh.^2];
            names={'NVR_Tx' 'NVR_Ty' 'NVR_Tz' 'NVR_Rx' 'NVR_Ry' 'NVR_Rz' ...
                   'NVR_Tx2' 'NVR_Ty2' 'NVR_Tz2' 'NVR_Rx2' 'NVR_Ry2' 'NVR_Rz2' ...
                   'NVR_Thx' 'NVR_Thy' 'NVR_Thz' 'NVR_Rhx' 'NVR_Rhy' 'NVR_Rhz' ...
                   'NVR_Thx2' 'NVR_Thy2' 'NVR_Thz2' 'NVR_Rhx2' 'NVR_Rhy2' 'NVR_Rhz2' };
        case 2 % Only linear
            mh=[0 0 0 0 0 0;m(1:end-1,:)];
            reg=[m mh];
            names={'NVR_Tx' 'NVR_Ty' 'NVR_Tz' 'NVR_Rx' 'NVR_Ry' 'NVR_Rz' ...
                   'NVR_Thx' 'NVR_Thy' 'NVR_Thz' 'NVR_Rhx' 'NVR_Rhy' 'NVR_Rhz' };
        case 3 % No history
            reg=[m m.^2];
            names={'NVR_Tx' 'NVR_Ty' 'NVR_Tz' 'NVR_Rx' 'NVR_Ry' 'NVR_Rz' ...
                   'NVR_Tx2' 'NVR_Ty2' 'NVR_Tz2' 'NVR_Rx2' 'NVR_Ry2' 'NVR_Rz2' };
        case 4 % Only linear, no history
            reg=[m];
            names={'NVR_Tx' 'NVR_Ty' 'NVR_Tz' 'NVR_Rx' 'NVR_Ry' 'NVR_Rz' };
        end
        
        %use regressors within a SVD subspace
        if exist('job.NVR_motion_svd') && job.NVR_motion_svd{1}>0
           %remove mean+drift
%            reg=reg-repmat(mean(reg,1),size(reg,1),1);
           reg=reg-X/(X'*X)*X'*reg;
           %Normalize in blocks of three (same unit translation, rotation etc.)
           vreg=var(reg,[],1);
           for i=1:3:length(vreg)
               reg(:,i:i+2)=reg(:,i:i+2)/sqrt(sum(vreg(i:i+2))+eps);
           end
           [Usvd,Ssvd,Vsvd]=svd(reg,'econ');
           reg=Usvd(:,1:sess.NVR.NVR_motion_svd{1});
           names=[];
           for ii=1:sess.NVR.NVR_motion_svd{1}
               names=[names {sprintf('NVR_movement SVD%d',ii)}];
           end
        end
        
        
        X = [X reg];
        rnames = [rnames names];
    end
    
    if isfield(job,'despikereg') && job.despikereg & exist('SP','var')
        if all(isfinite(SP))
            X=[X SP];
            rnames = [rnames 'SpikePercentage'];
        else
            warning('Wavelet despike did not report percentage correctly - not including spike percentage in filter')
        end
    end

   
    if isfield(job,'NVR_mask_rep') && length(job.NVR_mask_rep)>0
        try
            XYZ;
        catch
            [Y1, XYZ]=spm_read_vols(Vin(1));
            clear Y1
            XYZ=[XYZ;ones(1,size(XYZ,2))];
        end
    for ii=1:length(job.NVR_mask_rep)
        NVRdata=job.NVR_mask_rep{ii};
        
        if isfield(NVRdata,'NVR_mask')
            NVRdata=NVRdata.NVR_mask;
            V_mask=spm_vol(NVRdata.NVR_mask_file{1});
            XYZ_mask=V_mask.mat\XYZ;
            Y_mask=spm_sample_vol(V_mask,XYZ_mask(1,:),XYZ_mask(2,:),XYZ_mask(3,:),0);
            Y_mask(~all(isfinite(Y))|var(Y)==0)=0;
            if NVRdata.NVR_mask_erode>0
                Y_mask=reshape(Y_mask>NVRdata.NVR_mask_th,Vin(1).dim);
                [xgrid, ygrid, zgrid] = meshgrid(-NVRdata.NVR_mask_erode:NVRdata.NVR_mask_erode);
                ball = (sqrt(xgrid.^2 + ygrid.^2 + zgrid.^2) <= NVRdata.NVR_mask_erode);
                Y_mask=imerode(Y_mask,ball);
                %Y_mask=spm_erode(double(Y_mask),double(ball)); %doesnt work with non standard kernel
                Y_mask=Y_mask(:);
                idx=find(Y_mask);
            elseif NVRdata.NVR_mask_erode<0
                fprintf('Erode using spm_erode with standard kernel %i times, ',floor(-NVRdata.NVR_mask_erode))
                Y_mask=reshape(Y_mask>NVRdata.NVR_mask_th,Vin(1).dim);
                for jj=1:floor(-NVRdata.NVR_mask_erode)
                   Y_mask=spm_erode(double(Y_mask));
                end
                Y_mask=Y_mask(:);
                idx=find(Y_mask);
            else
                idx=find(Y_mask>NVRdata.NVR_mask_th);
            end
        elseif isfield(NVRdata,'NVR_point')
            NVRdata=NVRdata.NVR_point;
            dXYZ=sqrt(sum(bsxfun(@minus,XYZ(1:3,:),NVRdata.NVR_point_xyz(:)).^2));
            if NVRdata.NVR_point_r==0
                [minr idx]=min(dXYZ);
                if ~isfinite(Y(:,idx))
                    idx=[];
                end
            elseif NVRdata.NVR_point_r>0
                idx=find(dXYZ<=NVRdata.NVR_point_r)&all(isfinite(Y))&var(Y)>0;
            end
            clear dXYZ
        end
        if length(idx)>0
            fprintf('NVR data mask contains %d voxels.\n',length(idx))
            Y_mask=Y(:,idx);
            if NVRdata.NVR_mask_svd==0
                %Y_mask=Y_mask(:,all(isfinite(Y_mask)));
                NVRv=mean(Y_mask,2);
                NVRv=NVRv-mean(NVRv);
            elseif NVRdata.NVR_mask_svd>0
                %Y_mask = Y_mask(:,all(isfinite(Y_mask)));
                [Um Sm Vm]=svd(bsxfun(@minus,Y_mask,mean(Y_mask)),'econ');
                NVRv=Um(:,1:NVRdata.NVR_mask_svd);
            end
            X = [X NVRv];
            for jj=1:size(NVRv,2)
                rnames = [rnames {sprintf('NVR_data_%d',ii)}];
            end
            if NVRdata.NVR_deriv>0
                for jj=1:size(NVRv,2)
                    X = [X [zeros(1,1);diff(NVRv(:,jj))]];
                    rnames = [rnames {sprintf('dNVR_data_%d',ii)}];
                end
            end
            Vtmp=Vin(1);
            Vtmp.fname=fullfile(outdir,sprintf('regmask_%02i.nii',ii));
            Vtmp.dt=[2 0];
            Ytmp=zeros(Vtmp.dim);
            Ytmp(idx)=1;
            spm_write_vol(Vtmp,Ytmp);
        else
            fprintf('NVR data mask contains no voxels.\n')
        end
    end
    end

% Siemens
if isfield(job,'br')  && ~isempty(job.br) && ~isempty(job.br.physsiemens) 
    
    TRACETYPE       ='SIEMENS';
%    PULSE           =job.br.pulseon; %'ON'/'OFF'
%    RESP            =job.br.respon; %'ON'/'OFF' 
%    ECG             =job.br.ecgon; %'ON'/'OFF'
    RVT             =job.rvton; %'ON'/'OFF'
    RESPXECG        ='OFF';
    RESPXPULSE      ='OFF';
    PULSE_ORDER     =job.corder; %cardiac expansion order
    RESP_ORDER      =job.rorder; %respiratory expansion order
    ECG_ORDER       =job.ecgorder; %ecg expansion order
    RESPXPULSE_ORDER='OFF';
    RESPXECG_ORDER  ='OFF';
    START_STAMP     ='MG_START';
    STOP_STAMP      ='MG_STOP';
    DUMMY_VOLUMES   = job.br.ndummies;
    DICOM_FILE      ='';
    PHYS_FILE       =job.br.physsiemens{1}; 
    PHYS_DATE.YEAR_IDX=4:7; PHYS_DATE.MONTH_IDX=8:9; PHYS_DATE.DAY_IDX=10:11; PHYS_DATE.HOUR_IDX=13:14;
    PHYS_DATE.MINS_IDX=15:16; PHYS_DATE.SEC_IDX=17:18;
    TR              =job.RT; 
    NUM_VOLUMES     =T; %length(spm_vol(strvcat(job.input));
    NUM_SLICES      =job.nslices; 
    REF_SLICE       =job.sonset; 
    RVT_DELAY       =job.RT; % TR
    FIND_PULSEPEAKS ='OFF';
    FIND_ECGPEAKS   ='OFF';
    FIND_RESPPEAKS  ='OFF';
    FS_RESP         =[];
    FS_PULS         =[];
    FS_ECG          =[];
    PULSEPEAKS_INTERVAL =0.33; % if pulse above 180bpm is considered abnormal
    ECGPEAKS_INTERVAL   =0.4;  % tricky because ECG has more peaks
    RESPPEAKS_INTERVAL  =0.8;  % resp above 48 bpm is considered abnormal
    LP_PULSE        =job.br.lp_pulseon; %'ON'/'OFF'
    LP_ECG          =job.br.lp_ecgon;   %'ON'/'OFF'
    LP_RESP         =job.br.lp_respon;  %'ON'/'OFF'
    LP_PULSE_FSTOP  =8;                 % stopband in Hz
    LP_ECG_FSTOP    =8;                 % stopband in Hz
    LP_RESP_FSTOP   =11;                % stopband in Hz
   
%     % choose ON for physfiles if they are in the directory, otherwise OFF
%     [PHYS_DIR,PHYS_NAME] = fileparts(PHYS_FILE);
%     dir=fullfile(PHYS_DIR,PHYS_NAME);
%     status=system('dir *.resp');
%     if status == 0
%         RESP='ON';
%     else
%         RESP='OFF';
%     end
%     status=system('dir *.puls');
%     if status == 0
%         PULSE='ON';
%     else
%         PULSE='OFF';
%     end
%     status=system('dir *.ecg');
%     if status == 0
%         ECG='ON';
%     else
%         ECG='OFF';
%     end
    
% deleted: 'ECG',ECG,'PULSE',PULSE,'RESP',RESP,
    NVRstruct=make_NVR('TRACETYPE',TRACETYPE,'RESPXECG',RESPXECG,'RESPXPULSE',RESPXPULSE,...
    'DUMMY_VOLUMES',DUMMY_VOLUMES,'TR',TR,'NUM_VOLUMES',NUM_VOLUMES,'NUM_SLICES',NUM_SLICES,'REF_SLICE',REF_SLICE,...
    'ECG_ORDER',ECG_ORDER,'PULSE_ORDER',PULSE_ORDER,'RESP_ORDER',RESP_ORDER,'RVT_DELAY',RVT_DELAY,...
    'START_STAMP',START_STAMP,'STOP_STAMP',STOP_STAMP,'PHYS_FILE',PHYS_FILE,'DICOM_FILE',DICOM_FILE,...
    'PHYS_DATE',PHYS_DATE,'FS_PULS',FS_PULS,'FS_RESP',FS_RESP,'FS_ECG',FS_ECG,'RVT',RVT,...
    'FIND_PULSEPEAKS',FIND_PULSEPEAKS,'FIND_ECGPEAKS',FIND_ECGPEAKS,'FIND_RESPPEAKS',FIND_RESPPEAKS,...
    'PULSEPEAKS_INTERVAL',PULSEPEAKS_INTERVAL,'ECGPEAKS_INTERVAL',ECGPEAKS_INTERVAL,'RESPPEAKS_INTERVAL',RESPPEAKS_INTERVAL,...
    'LP_PULSE',LP_PULSE,'LP_ECG',LP_ECG,'LP_RESP',LP_RESP,'LP_PULSE_FSTOP',LP_PULSE_FSTOP,...
    'LP_ECG_FSTOP',LP_ECG_FSTOP,'LP_RESP_FSTOP',LP_RESP_FSTOP,'RESPXPULSE_ORDER',RESPXPULSE_ORDER,'RESPXECG_ORDER',RESPXECG_ORDER);

     pregs1=NVRstruct.pulse_retroicor;
     pregs2=NVRstruct.resp_retroicor;
     pregs3=NVRstruct.ecg_retroicor;
     pregs4=NVRstruct.rvt; %%%
     pregs=[pregs1 pregs2 pregs3 pregs4];

     %rnames and X from NVR-section
     X=[X pregs];
     for i=1:size(pregs,2)
        rnames=[rnames {sprintf('physio_%i',i)}];
     end
end

% Philips
if isfield(job,'physfile') && ((iscell(job.physfile) && ~isempty(job.physfile{1})) || (~isempty(job.physfile)))
addpath(physdir);

physio      = tapas_physio_new();
log_files   = physio.log_files;
thresh      = physio.thresh;
sqpar       = physio.sqpar;
model       = physio.model;
verbose     = physio.verbose;

log_files.vendor            = 'Philips';
log_files.cardiac           = job.physfile;      
log_files.respiration       = job.physfile;  
if job.nslices==0
    job.nslices=Vin(1).dim(3);
end
sqpar.Nslices           = job.nslices;
sqpar.NslicesPerBeat    = job.nslices;
sqpar.TR                = job.RT;

sqpar.Ndummies          = 0;
sqpar.Nscans            = T;
if job.sonset==0
    job.sonset=round(job.nslices/2);
end
sqpar.onset_slice       = job.sonset;

% 2.2. Counting scans and dummy volumes from beginning of run, i.e. logfile,
%      includes counting of preparation gradients        
% (Uncomment the following line to execute) 
%sqpar.Nprep = 3;


%% 3. Order of RETROICOR-expansions for cardiac, respiratory and
%% interaction terms. Option to orthogonalise regressors

RVT=job.rvton;
 if strcmp(RVT,'ON') %%%
     model.type='RETROICOR_RVT';
 else
     model.type='RETROICOR';
 end

%model.type = 'RETROICOR_RVT';
%model.type = 'none';

model.order = struct('c',job.corder,'r',job.rorder,'cr',job.crorder, 'orthogonalise', 'none');
model.input_other_multiple_regressors = ''; % either txt-file or mat-file with variable R
model.output_multiple_regressors = '';


%% 4. Define Gradient Thresholds to Infer Gradient Timing (Philips only) %should be setable??
				%changed the thresholds from 700/1200, KWA 060217
				thresh.scan_timing = struct('method', 'gradient_log', 'zero', 700, 'slice', 1200, 'vol', [], ...
 'grad_direction', 'y');
thresh.scan_timing.vol = [];
%thresh.scan_timing.vol_spacing = 0.07; % in seconds


%% 5. Define which Cardiac Data Shall be Used

%% 5.1. Using heart beat events logged prospectively during scanning instead also setable?
thresh.cardiac.modality = 'OXY'; % 'ECG','ECG_raw', 'OXY' or 'OXYGE' (for pulse oximetry)

%% 5.2. Using ECG time curve to detect heartbeat events, via a chosen or
%% saved reference R-peak
thresh.cardiac.initial_cpulse_select.min = 1;
thresh.cardiac.initial_cpulse_select.method = 'auto_template'; % 'load_from_logfile', 'manual' or 'load' (from previous manual run)


%% 6. Output Figures to be generated

verbose.level = 0;
% 0 = none; 
% 1 = main plots (default); 
% 2 = debugging plots: for missed slice/volume events, missed heartbeats, 1D time series of created regressors
% 3 = all plots, incl. cardiac/respiratory phase estimation,
%     slice-to-volume assignment

verbose.fig_output_file = fullfile(outdir,'PhysIO_output.jpg');


%% 7. Run the main script with defined parameters

physio.log_files    = log_files;
physio.thresh       = thresh;
physio.sqpar        = sqpar;
physio.model        = model;
physio.verbose      = verbose;

% physio = tapas_physio_new('manual_peak_select', physio);
% physio.thresh.cardiac.posthoc_cpulse_select.method = 'load';

[physio_out, pregs, ons_secs] = tapas_physio_main_create_regressors(physio);
%pregs=[];
X=[X pregs];
for i=1:size(pregs,2)
    rnames=[rnames {sprintf('physio_%i',i)}];
end
rmpath(physdir);

% Plot physio-logfiles
[z{1:10}]=textread(physio.log_files.respiration{1},'%d %d %d %d %d %d %d %d %d %d','commentstyle','shell');
y = cell2mat(z);
    
figure
picfile=fullfile(outdir,[outname '_physio.pdf']);
subplot(2,1,1)
plot(y(:,6))
title('Respiration')

subplot(2,1,2)
plot(y(:,5))
title('Cardiac')
saveas(gcf,picfile);
%close(figure(1))

end


%Remove volumes (exclude)
if not(isempty(job.exvol))
    try
        motion_exclude(job.exvol(iii))=1;
    catch
        warning('Failed during volume exclude.')
    end
end
exlist=find(motion_exclude);
for iiii=1:length(exlist)
    iii=exlist(iiii);
    tmpv=zeros(T,1);
    tmpv(iii)=1;
    X = [X tmpv];
    rnames = [rnames {sprintf('NVR_exscan_%i',iii)}];
end

if not(isempty(job.NVR_extra))
%Add extra nuisance regressors
    extra=job.NVR_extra;
    X = [X extra];
    exnames=[];
    for ij=1:size(extra,2)
        exnames{ij}=sprintf('NVR_extra%d',ij);
    end
    rnames = [rnames exnames];
%    nuisance_length(i)=nuisance_length(i)+size(extra,2);
end

X(:,2:end)=bsxfun(@minus,X(:,2:end),mean(X(:,2:end)));
stdX=1./std(X(:,2:end));
stdX(~isfinite(stdX))=1;
X(:,2:end)=bsxfun(@times,X(:,2:end),stdX);

NVRdebug=true;
if NVRdebug
    h=figure;
    [adir afn]=fileparts(Vin(1).fname);
    title(sprintf('Motion diagnostics %s',adir))
    ttmp=1:T;%linspace(0,(SPM.nscan(i)-1)*SPM.xY.RT,SPM.nscan(i));
    if exist('dvars')
        maxdvars=1.05*max([dvars;job.DVARS]);
        if exist('FD')
            subplot(2,1,1)
        else
            subplot(3,1,1)
        end
        for ii=1:length(ttmp)
            if exclude_DVARS(ii)
                fh=area([ii-.5 ii+.5],[maxdvars maxdvars]);
                set(fh,'FaceColor',[1 0.5 0.5]);
                set(fh,'LineStyle','none');
            else
                fh=area([ii-.5 ii+.5],[maxdvars maxdvars]);
                set(fh,'FaceColor',[0.7 0.7 0.7]);
                set(fh,'LineStyle','none');
            end
            hold on
        end
        plot(ttmp,dvars,'k','LineWidth',1)
        if isfinite(job.DVARS)
        plot([min(ttmp) max(ttmp)],repmat(job.DVARS,1,2),'k--','LineWidth',1)
        end
        xlim([min(ttmp) max(ttmp)])
        ylim([0 maxdvars])
        xlabel('scans')
        ylabel(sprintf('%% signal change'))
        try
            Xr=rank(X);
            title(sprintf('DVARS %d vols, %d vols total, filter rank: %i, Res dof: ~%i.',sum(exclude_DVARS),sum(motion_exclude),Xr,T-Xr))
        catch
            title(sprintf('DVARS %d vols, %d vols total',sum(exclude_DVARS),sum(motion_exclude)))
        end
    end
        if exist('FD')
            subplot(2,1,2)
        maxrel=1.05*max([FD;job.NVR_motion_threshold_rel]);
        for ii=1:length(ttmp)
            if exclude_rel(ii)
                fh=area([ii-.5 ii+.5],[maxrel maxrel]);
                set(fh,'FaceColor',[1 0.5 0.5]);
                set(fh,'LineStyle','none');
            else
                fh=area([ii-.5 ii+.5],[maxrel maxrel]);
                set(fh,'FaceColor',[0.7 0.7 0.7]);
                set(fh,'LineStyle','none');
            end
            hold on
        end
        plot(ttmp,FD,'k','LineWidth',1)
        if isfinite(job.NVR_motion_threshold_rel)
            plot([min(ttmp) max(ttmp)],repmat(job.NVR_motion_threshold_rel,1,2),'k--','LineWidth',1)
        end
        xlim([min(ttmp) max(ttmp)])
        ylim([0 maxrel])
        xlabel('scans')
        ylabel('mm')
        title(sprintf(' Framewise displacement excluded %d volumes',sum(exclude_rel)))
        else
    if exist('relmotion')
        subplot(3,1,2)
        maxrel=1.05*max([relmotion;job.NVR_motion_threshold_rel]);
        for ii=1:length(ttmp)
            if exclude_rel(ii)
                fh=area([ii-.5 ii+.5],[maxrel maxrel]);
                set(fh,'FaceColor',[1 0.5 0.5]);
                set(fh,'LineStyle','none');
            else
                fh=area([ii-.5 ii+.5],[maxrel maxrel]);
                set(fh,'FaceColor',[0.7 0.7 0.7]);
                set(fh,'LineStyle','none');
            end
            hold on
        end
        plot(ttmp,[0;relmotion],'k','LineWidth',1)
        if isfinite(job.NVR_motion_threshold_rel)
            plot([min(ttmp) max(ttmp)],repmat(job.NVR_motion_threshold_rel,1,2),'k--','LineWidth',1)
        end
        xlim([min(ttmp) max(ttmp)])
        ylim([0 maxrel])
        xlabel('scans')
        ylabel('mm')
        title(sprintf(' Rel. motion excluded %d volumes',sum(exclude_rel)))
    end
    if exist('absmotion')
        subplot(3,1,3)
        maxabs=1.05*max([absmotion;job.NVR_motion_threshold_abs]);
        for ii=1:length(ttmp)
            if exclude_abs(ii)
                fh=area([ii-.5 ii+.5],[maxabs maxabs]);
                set(fh,'FaceColor',[1 0.5 0.5]);
                set(fh,'LineStyle','none');
            else
                fh=area([ii-.5 ii+.5],[maxabs maxabs]);
                set(fh,'FaceColor',[0.7 0.7 0.7]);
                set(fh,'LineStyle','none');
            end
            hold on
        end
    
        
        plot(ttmp,absmotion,'k','LineWidth',1)
        if isfinite(job.NVR_motion_threshold_abs)
            plot([min(ttmp) max(ttmp)],repmat(job.NVR_motion_threshold_abs,1,2),'k--','LineWidth',1)
        end
        xlim([min(ttmp) max(ttmp)])
        ylim([0 maxabs])
        xlabel('scans')
        ylabel('mm')
        title(sprintf(' Abs. motion excluded %d volumes',sum(exclude_abs)))
    end
    end
    try
        saveas(h,fullfile(outdir,sprintf('%s_mdiag.pdf',outname)));
    catch
        fprintf('Cannot write pdf file %s',sprintf('%s_mdiag.pdf',outname));
    end
    close(h);


if exist('NVRstruct')
h=figure;
title(sprintf('Phys diagnostics %s',adir))
if ~isempty(NVRstruct.pulse_onsets)
    try
        subplot(3,1,1)
        pulserate=60./diff(NVRstruct.pulse_onsets);
        plot(NVRstruct.pulse_onsets(2:end),pulserate,'k','linewidth',1);
        xlim([0 max(NVRstruct.pulse_onsets)])
        ylim([0 max([120 max(pulserate)])])
        title('Pulse oximetry')
        xlabel('time [s]')
        ylabel('bpm')
    end
end

if ~isempty(NVRstruct.resp_onsets)
    try
        subplot(3,1,2)
        resptime=60./diff(NVRstruct.resp_onsets);
        plot(NVRstruct.resp_onsets(2:end),resptime,'k','linewidth',1);
        xlim([0 max(NVRstruct.resp_onsets)])
        ylim([0 max([20 max(resptime)])])
        title('Respiration')
        xlabel('time [s]')
ylabel('rpm')
    end
end
if ~isempty(NVRstruct.ecg_onsets)
    try
        subplot(3,1,3)
        ecgrate=60./diff(NVRstruct.ecg_onsets);
        plot(NVRstruct.ecg_onsets(2:end),ecgrate,'k','linewidth',1);
        xlim([0 max(NVRstruct.ecg_onsets)])
        ylim([0 max([120 max(ecgrate)])])
        title('ECG')
        xlabel('time [s]')
        ylabel('bpm')
    end
end
saveas(h,fullfile(adir,sprintf('NVR_onsets.pdf')));
close(h);
end

end
if job.savefilter
    dofres=T-rank(X);
end


if job.savedata

%residual forming matrix
[Ux Sx Vx]=svd(X,'econ'); %prewhiten to avoid solution being badly scaled
%here it might be a good idea to remove close to zero singular vectors
if any(diag(Sx)<(eps*mean(diag(Sx))))
    inc=diag(Sx)>=(eps*mean(diag(Sx)));
    disp(sprintf('excluding %i singular regresors',sum(diag(Sx)<(eps*mean(diag(Sx))))));
    Ux=Ux(:,inc);
    Sx=Sx(inc,inc);
end
R=eye(T)-Ux*Ux';
Yr=R*Y;

testing=true;
if testing
   RSS=sum(Yr.^2);
   dofr=(size(Y,1)-rank(X));
   names={'NVR_Tx' 'NVR_Ty' 'NVR_Tz' 'NVR_Rx' 'NVR_Ry' 'NVR_Rz' ...
                   'NVR_Tx2' 'NVR_Ty2' 'NVR_Tz2' 'NVR_Rx2' 'NVR_Ry2' 'NVR_Rz2' ...
                   'NVR_Thx' 'NVR_Thy' 'NVR_Thz' 'NVR_Rhx' 'NVR_Rhy' 'NVR_Rhz' ...
                   'NVR_Thx2' 'NVR_Thy2' 'NVR_Thz2' 'NVR_Rhx2' 'NVR_Rhy2' 'NVR_Rhz2' };
   mcols=zeros(size(X,2),1);
   if ~length(rnames)==size(X,2)
       error('regressor names not found.')
   end
   for i=1:size(X,2)
       for j=1:length(names)
           if strcmp(rnames{i},names{j})
               mcols(i)=1;
               break
           end
       end
   end
   if any(mcols)
       X_m=X(:,mcols==0);
       [Uxm Sxm Vxm]=svd(X_m,'econ');
       if any(diag(Sxm)<(eps*mean(diag(Sxm))))
            inc=diag(Sxm)>=(eps*mean(diag(Sxm)));
            Uxm=Uxm(:,inc);
            Sxm=Sxm(inc,inc);
       end
       RSS_m=sum(((eye(T)-Uxm*Uxm')*Y).^2);
       dof_m=rank(X)-rank(X_m);
       RSS_m=dofr/dof_m*((RSS_m-RSS+eps)./(RSS+eps)); %F
       Vtmp=Vin(1);
       Vtmp.fname=fullfile(outdir,sprintf('motion_%s%s.nii',job.prefix,outname));
       Vtmp.dt=[16 0];
       try
            Vtmp.descrip=sprintf('F(%i,%i) - F_th_0.001=%.3f',dof_m,dofr,finv(1-0.001,dof_m,dofr));
       catch
           Vtmp.descrip=sprintf('F(%i,%i) - F_th_0.001',dof_m,dofr);
       end
       spm_write_vol(Vtmp,reshape(RSS_m,Vtmp.dim));
       clear X_m
       clear RSS_m
   end
   
   names={'physio_'};
   pcols=zeros(size(X,2),1);
   if ~length(rnames)==size(X,2)
       error('regressor names not found.')
   end
   for i=1:size(X,2)
       for j=1:length(names)
           if ~isempty(regexp(rnames{i},names{j}))
               pcols(i)=1;
               break
           end
       end
   end
   if any(pcols)
       X_p=X(:,pcols==0);
       RSS_p=sum(((eye(T)-X_p/(X_p'*X_p)*X_p')*Y).^2);
       dof_p=rank(X)-rank(X_p);
       RSS_p=dofr/dof_p*((RSS_p-RSS+eps)./(RSS+eps)); %F
       Vtmp=Vin(1);
       Vtmp.fname=fullfile(outdir,sprintf('phys_%s%s.nii',job.prefix,outname));
       Vtmp.dt=[16 0];
       Vtmp.descrip=sprintf('F(%i,%i) - F_th_0.001=%.3f',dof_m,dofr,finv(1-0.001,dof_m,dofr));
       spm_write_vol(Vtmp,reshape(RSS_p,Vtmp.dim));
       clear X_p
       clear RSS_p
    end
   
   clear RSS 
end



if job.addmean
    Yr=bsxfun(@plus,Yr,mean(Y));
end
clear Y
if isfield(job,'despike') && job.despike && ~job.despikefirst
    [Yr,noise,SP]=runWaveletDespike(Yr,wdOpts);
    clear noise
end

    Yr=reshape(Yr',Vin(1).dim(1),Vin(1).dim(2),Vin(1).dim(3),T);

    ind  = cat(1,Vin.n);
    N    = cat(1,Vin.private);



    fname=fullfile(outdir,sprintf('%s%s.nii',job.prefix,outname));

    dt=0;%data type 0: keep same

    %-Input: dtype
    %--------------------------------------------------------------------------
    if dt == 0
        dt = Vin(1).dt(1);
    end

    %-Set scalefactors and offsets
    %==========================================================================
    d = cat(1,Vin.dt); d = d(:,1);
    s = cat(2,Vin.pinfo); s = s(1,:);
    o = cat(2,Vin.pinfo); o = o(2,:);

    %-Reuse parameters of input images if same scalefactor, offset and datatype
    %--------------------------------------------------------------------------
    if length(unique(s)) == 1 && length(unique(o)) == 1 ...
            && length(unique(d)) == 1 && d(1) == dt
        sf  = Vin(1).pinfo(1);
        off = Vin(1).pinfo(2);
    else

        dmx  = spm_type(dt,'maxval');
        dmn  = spm_type(dt,'minval');

        %-Integer datatypes: scale to min/max of overall data
        %----------------------------------------------------------------------
        if isfinite(dmx)
            mx      = -Inf;
            mn      = Inf;
            for i=1:numel(Vin)
                dat = Vin(i).private.dat(:,:,:,ind(i,1),ind(i,2));
                dat = dat(isfinite(dat));
                mx  = max(mx,max(dat(:)));
                mn  = min(mn,min(dat(:)));
                spm_progress_bar('Set',i);
            end
            spm_progress_bar('Clear');
            if isempty(mx), mx = 0; end
            if isempty(mn), mn = 0; end

            if mx~=mn
                if dmn < 0
                    sf = max(mx/dmx,-mn/dmn);
                else
                    sf = mx/dmx;
                end
                off    = 0;
            else
                sf     = mx/dmx;
                off    = 0;
            end

        %-floating precison: no scaling
        %----------------------------------------------------------------------
        else
            sf         = 1;
            off        = 0;
        end
    end

    %-Create and write 4D volume image
    %==========================================================================
    spm_unlink(fname);

    ni         = nifti;
    ni.dat     = file_array(fname,...
                            [Vin(1).dim numel(Vin)],...
                            [dt spm_platform('bigend')],...
                            0,...
                            sf,...
                            off);
    ni.mat     = N(1).mat;
    ni.mat0    = N(1).mat;
    ni.descrip = '4D filtered image';

    create(ni);
    ni.dat(:,:,:,:) = Yr(:,:,:,:);
    for i=1:size(ni.dat,4)
        spm_get_space([ni.dat.fname ',' num2str(i)], Vin(1).mat);
    end
    job.out=ni.dat.fname;
end
if job.savefilter
    if ~exist('SP','var')
        SP=[];
    end
    if ~exist('R','var')
        R=[];
    end
    if ~exist('dvars','var')
        dvars=[];
    end
    if ~exist('FD','var')
        FD=[];
    end
    dofres = T-rank(X);
    %dofres=rank(Y);
    save(fullfile(outdir,sprintf('%s_regs.mat',outname)),'X','R','SP','dvars','FD','rnames','dofres');
end

disp('Temporal filtering - Done');

cd(sdir)

end
function dep = vout_filter(job)

dep=cfg_dep;
dep.sname='Filtered images';
dep.src_output   = substruct('.','out');
dep.tgt_spec     = cfg_findspec({{'filter','image','strtype','e'}});

end

function out=run_despike(in,out,opts)
%run
%uncompress output
end

function [clean,noise,SP] = runWaveletDespike(ts,Opts)
% Adapted from original function by Ameera X Patel
% FUNCTION:     WaveletDespike -- Wavelet Despikes time series, as
%                                 described in:
%
% Patel, AX. et al (2014). A wavelet method for modeling and despiking 
% motion artifacts from resting-state fMRI time serires. NeuroImage.
%
%
% USAGE:        WaveletDespike(Inii,Opref,varargin)
%
% Inputs:       Inii      -- NIfTI file (3D+t dataset) containing time
%                            series to despike.
%               Opref     -- Output prefix for despiked NIfTI files and
%                            Spike Percentage (if specified).
%
%               Additional Input Options:
%               (These must be specific as MATLAB string-value pairs.)
%
%               wavelet   -- Wavelet to use for wavelet transform. Input
%                            must be a string containing one of the 
%                            following:
%                            'd4','d6','d8','d10','d12','d14','d16','d18',
%                            'd20','la8','la10','la12','la14','la16',
%                            'la18','la20','bl14','bl18','bl20','c6','c12',
%                            'c18','c24','haar'. [Default='d4'].
%               threshold -- Threshold for maximal and minimal wavelet
%                            coefficients. [Default=10].
%               boundary  -- Boundary condition to apply. Input must be a
%                            string containing one of the following: 
%                            'circular','reflection'.
%                            [Default='reflection'].
%               chsearch  -- Rules for identifying maxima and minima 
%                            chains. Input must be a string containing one 
%                            of the following: 'conservative', 'moderate',
%                            'harsh'. [Default='moderate'].
%               nscale    -- Method for computing number of scales. Input
%                            must be a string containing one of the
%                            following: 'conservative','liberal','extreme'.
%                            [Default: 'liberal'].
%               compress  -- Binary flag indicating whether to compress out
%                            non-brain regions, 1, or not, 0, from input 
%                            NIfTI file before Wavelet Despiking. This 
%                            saves RAM and reduces runtime. [Default=1].
%               sp        -- Binary flag indicating whether to output the
%                            Spike Percentage for the dataset.
%                            [Default=1].
%               LimitRAM  -- Specify an upper bound of RAM usage (in Giga-
%                            bytes). Default is to use all available RAM.
%               verbose   -- Binary flag indicating whether to display
%                            incremental output from the algorithm, 1, or
%                            not, 0.
% 
% Outputs:      This function will write the following files to the current
%               directory:
%
%               Opref_wds.nii.gz   - Wavelet Despiked time series.
%               Opref_noise.nii.gz - Noise components removed in Wavelet
%                                    Despiking.
%               Opref_SP.txt       - The Spike Percentage (if specified at
%                                    input).
%
% EXAMPLE:      WaveletDespike('rest.nii.gz','rest','wavelet','la8',...
%                   'LimitRAM',5)
%
% AUTHOR:       Ameera X Patel
% CREATED:      26-12-2013
%   
% CREATED IN:   MATLAB 7.13
%
% REVISION:     19
%
% COPYRIGHT:    Ameera X Patel (2013, 2014), University of Cambridge
%
% TOOLBOX:      BrainWavelet Toolbox v1.1

% ID: wavcorr.m 19 07-02-2014 BWTv1.1 axpatel


%% check nopts / parse extra opts
scriptdir=fullfile(spm('dir'),'toolbox','BrainWavelet_1.1');
addpath(fullfile(scriptdir,'BWT'));
addpath(fullfile(scriptdir,'third_party/cprintf'));
addpath(fullfile(scriptdir,'third_party/NIfTI'));
addpath(fullfile(scriptdir,'third_party/wmtsa/dwt'));
addpath(fullfile(scriptdir,'third_party/wmtsa/utils'));

% DefaultOpts=struct('wavelet','d4','threshold',10,'boundary','reflection',...
%     'chsearch','moderate','nscale','liberal','compress',1,'SP',1,...
%     'LimitRAM',0,'verbose',1);
% Opts=parseInOpts(DefaultOpts,varargin);
% 
% %% check inputs
% 
% wavelets={'d4','d6','d8','d10','d12','d14','d16','d18','d20',...
%       'la8','la10','la12','la14','la16','la18','la20',...
%       'bl14','bl18','bl20','c6','c12','c18','c24','haar'};
% boundaries={'periodic','circular','reflection'};
% ok_chtype={'conservative','moderate','harsh'};
% scaleopts={'conservative','liberal','extreme'};

% err=struct();
% err(1).inp=chkintype(Inii,'char',fname);
% err(2).inp=chkintype(Opref,'char',fname);
% err(3).inp=chkintype(Opts.wavelet,'char',fname,wavelets);
% err(4).inp=chkintype(Opts.threshold,'numeric',fname);
% err(5).inp=chkintype(Opts.boundary,'char',fname,boundaries);
% err(6).inp=chkintype(Opts.chsearch,'char',fname,ok_chtype);
% err(7).inp=chkintype(Opts.nscale,'char',fname,scaleopts);
% err(8).inp=chkintype(Opts.compress,'numeric',fname,{'0','1'});
% err(9).inp=chkintype(Opts.SP,'numeric',fname,{'0','1'});  
% err(10).inp=chkintype(Opts.LimitRAM,'numeric',fname);
% err(11).inp=chkintype(Opts.verbose,'numeric',fname',{'0','1'});
% if sum(cat(1,err.inp))>=1;
%    return
% end

%% load NIfTI, parse MemorySolver for BatchMode

t=cputime;

header;

[NX,NTS]=size(ts);

if strcmpi(Opts.boundary,'reflection');
    NXref=NX*2;
    [BatchMode,freeRAM]=MemorySolver(NTS,NXref,'LimitRAM',Opts.LimitRAM);
else 
    [BatchMode,freeRAM]=MemorySolver(NTS,NX,'LimitRAM',Opts.LimitRAM);
end

if isempty(BatchMode);
    return; 
end

%% run wavelet despiking in normal mode

if BatchMode==0
        [clean,noise,SP]=wdscore(ts,'wavelet',Opts.wavelet,'threshold',...
            Opts.threshold,'boundary',Opts.boundary,'chsearch',...
            Opts.chsearch,'nscale',Opts.nscale,'verbose',Opts.verbose);
end

%% run wavelet despiking in batch mode

if BatchMode>=1
    
    Ind=ParseInds(NTS,BatchMode);
    Batchstr=num2str(BatchMode);
    clean=zeros(NX,NTS);
    noise=zeros(NX,NTS);
    
    fprintf('Initialising Batch Mode ...\n')
	
    if Opts.LimitRAM==0
        fprintf('Using all available RAM. WARNING: your computer \n')
        fprintf('may be slow whilst this program is running.\n')
    else
        fprintf('Capping RAM usage at %s Gb ...\n',...
            num2str(freeRAM));
    end
        
    for ii=1:BatchMode
        
        fprintf('\nDespiking batch %s of %s.\n', num2str(ii),...
            Batchstr);
        
        tts=ts(:,Ind(1,ii):Ind(2,ii));
        

            [clean(:,Ind(1,ii):Ind(2,ii)),noise(:,Ind(1,ii):Ind(2,ii)),...
                SP(:,ii)]=wdscore(tts,'wavelet',Opts.wavelet,...
                'threshold',Opts.threshold,'boundary',Opts.boundary,...
                'chsearch',Opts.chsearch,'nscale',Opts.nscale,'verbose',...
                Opts.verbose);
            SP(:,ii)=SP(:,ii).*((Ind(2,ii)-Ind(1,ii)+1)/NTS);
        
    end
    
    if exist('SP','var')
        SPout=zeros(size(SP,1),1);
        for ii=1:size(SP,1)
            SPout(ii)=sum(SP(ii,isfinite(SP(ii,:))));
        end
        SP=SPout;
    end
end


% %% Write files
% 
% WriteOutNii(clean,strcat(Opref,'_wds.nii.gz'),Info);
% WriteOutNii(noise,strcat(Opref,'_noise.nii.gz'),Info);
% 
% if exist('SP','var')
%     dlmwrite(sprintf('%s_SP.txt',Opref),SP,' ');
% end

footer(t);


rmpath(fullfile(scriptdir,'BWT'));
rmpath(fullfile(scriptdir,'third_party/cprintf'));
rmpath(fullfile(scriptdir,'third_party/NIfTI'));
rmpath(fullfile(scriptdir,'third_party/wmtsa/dwt'));
rmpath(fullfile(scriptdir,'third_party/wmtsa/utils'));

end
%% Nuisance Variable Regression for Siemens
function NVRstruct=make_NVR(varargin)
  TRACETYPES=[{'GE' 'SIEMENS' 'IOP' 'CUSTOM'}];

  if nargin < 1
    help make_NVR  
    return
  end

%%%%%%%%%% Collect keywords and values from argument list %%%%%%%
   if (nargin> 0 & rem(nargin,2) ~= 0)
      error('Input must be pairs of key and corresponding value')
   end
  
   for i=1:2:length(varargin)
      Keyword = varargin{i};
      Value = varargin{i+1};
        if isstr(Keyword)
            if strcmp(upper(Keyword),'DEFAULTS_FILE')
                DEFAULTS_FILE=Value;
                run(Value)
            end
       end
   end
   
   
   for i = 1:2:length(varargin) % for each Keyword
      Keyword = varargin{i};
      Value = varargin{i+1};
      if ~isstr(Keyword)
         fprintf('Keywords must be strings')
         return
      end
      Keyword = upper(Keyword); % convert upper or mixed case to lower

%DEFAULTS_FILE
      if strcmp(Keyword,'DEFAULTS_FILE')
         display(['The following DEFAULTS_FILE is used: ' Value])

%TRACETYPE
      elseif strcmp(Keyword,'TRACETYPE')
         if ~isstr(Value)
            error(['TRACETYPE must be a string e.g. ' TRACETYPES])  
         else
            TRACETYPE= upper(Value);
            if ~sum(strcmp(TRACETYPE,TRACETYPES))
                error(['TRACETYPE: ' TRACETYPE 'is not yet supported, please send an email to one of the authors.'])  
            end
         end
         
% %EGG         
%       elseif strcmp(Keyword,'ECG')
%          if ~isstr(Value)
%            error('ECG value must be ON or OFF')
%          else 
%            Value = upper(Value);
%            if strcmp(Value,'ON') 
%               ECG = Value;
%               display('ECG is ON')
%            elseif strcmp(Value,'OFF'),
%               ECG = Value;
%               display('ECG is OFF')
%            else
%               error('ECG value must be ON or OFF')
%            end
%          end
% 
% %PULSE
%       elseif strcmp(Keyword,'PULSE')
%          if ~isstr(Value)
%            error('PULSE value must be ON or OFF')
%          else 
%            Value = upper(Value);
%            if strcmp(Value,'ON') 
%               PULSE = Value;
%               display('PULSE is ON')
%            elseif strcmp(Value,'OFF'),
%               PULSE = Value;
%               display('PULSE is OFF')
%            else
%               error('PULSE value must be ON or OFF')
%            end
%          end
%          
% %RESP
%        elseif strcmp(Keyword,'RESP')
%          if ~isstr(Value)
%            error('RESP value must be ON, OFF or ESTIMATE')
%          else 
%            Value = upper(Value);
%            if  strcmp(Value,'ON') 
%               RESP = Value; %added
%               display('RESP is ON') %added
%            elseif strcmp(Value,'OFF') | strcmp(Value,'ESTIMATE') %added
%                RESP = Value;
%            else
%               error('RESP value must be ON or OFF')
%            end
%          end

%RESPXECG
       elseif strcmp(Keyword,'RESPXECG')
         if ~isstr(Value)
           error('RESPXECG value must be ON, OFF')
         else 
           Value = upper(Value);
           if ( strcmp(Value,'ON') |strcmp(Value,'OFF'))
              RESPXECG = Value;
           else
              error('RESPXECG value must be ON or OFF')
           end
         end
         
%RESPXPULSE
       elseif strcmp(Keyword,'RESPXPULSE')
         if ~isstr(Value)
           error('RESPXPULSE value must be ON, OFF')
         else 
           Value = upper(Value);
           if ( strcmp(Value,'ON') |strcmp(Value,'OFF'))
              RESPXPULSE = Value;
           else
              error('RESPXPULSE value must be ON or OFF')
           end
         end
         
         
%NVR
       elseif strcmp(Keyword,'NVR')
         if ~isstr(Value)
           error('NVR value must be ON or OFF')
         else 
           Value = upper(Value);
           if ( strcmp(Value,'ON') |strcmp(Value,'OFF'))
              NVR = Value;
           else
              error('NVR value must be ON or OFF')
           end
         end
         
         
         
         
         
%DUMMY_VOLUMES        
        elseif strcmp(Keyword,'DUMMY_VOLUMES')
            if isstr(Value)
                Value=upper(Value)
                if ~strcmp(Value,'DEFAULT')
                    error('DUMMY_VOLUMES must be integer or DEFAULT')
                end
            elseif rem(Value,1)
                error('DUMMY_VOLUMES must be integer or DEFAULT')
            end
         DUMMY_VOLUMES=Value;

    
         
%TR
        elseif strcmp(Keyword,'TR')
            if isstr(Value)
                error('TR must be the repitition time in seconds')
            elseif ~isreal(Value)
                error('TR needs to be a real number')
            end
         TR=Value;
         
         
%NUM_VOLUMES
        elseif strcmp(Keyword,'NUM_VOLUMES')
            if isstr(Value)
                error('NUM_VOLUMES must be an integer')
            elseif rem(Value,1)
                error('NUM_VOLUMES must be an integer')
            end
         NUM_VOLUMES=Value;

%NUM_SLICES
        elseif strcmp(Keyword,'NUM_SLICES')
            if isstr(Value)
                error('NUM_SLICES must be an integer')
            elseif rem(Value,1)
                error('NUM_SLICES must be an integer')
            end
         NUM_SLICES=Value;


%REF_SLICE
        elseif strcmp(Keyword,'REF_SLICE')
         if isstr(Value)
             error('REF_SLICE must be an integer')
         elseif rem(Value,1)
             error('REF_SLICE must be an integer')
         end
         REF_SLICE=Value;
            
%ECG_ORDER
        elseif strcmp(Keyword,'ECG_ORDER')
         if isstr(Value)
             error('ECG_ORDER must be an integer')
         elseif rem(Value,1)
             error('ECG_ORDER must be an integer')
         end
         ECG_ORDER=Value;

%PULSE_ORDER
        elseif strcmp(Keyword,'PULSE_ORDER')
         if isstr(Value)
             error('PULSE_ORDER must be an integer')
         elseif rem(Value,1)
             error('PULSE_ORDER must be an integer')
         end
         PULSE_ORDER=Value;
         
%RESP_ORDER
        elseif strcmp(Keyword,'RESP_ORDER')
         if isstr(Value)
             error('RESP_ORDER must be an integer')
         elseif rem(Value,1)
             error('RESP_ORDER must be an integer')
         end
         RESP_ORDER=Value;
         
%RVT_DELAY
        elseif strcmp(Keyword,'RVT_DELAY')
            if isstr(Value)
                error('RVT_DELAY must be the repitition time in seconds (it can be a vector)')
            elseif ~isreal(Value)
                error('RVT_DELAY needs to be a real number or a real vector')
            end
         %TR=Value;
         RVT_DELAY=Value; %%!!added

%START_STAMP
      elseif strcmp(Keyword,'START_STAMP')
         START_STAMP=Value;

%STOP_STAMP
      elseif strcmp(Keyword,'STOP_STAMP')
         STOP_STAMP=Value;

%PHYS_FILE      
      elseif strcmp(Keyword,'PHYS_FILE')
         PHYS_FILE=Value;      

%DICOM_FILE
      elseif strcmp(Keyword,'DICOM_FILE')
         DICOM_FILE=Value;      

%PHYS_DIR      
%      elseif strcmp(Keyword,'PHYS_DIR')
%         PHYS_DIR=Value;      

%PHYS_DATE      
      elseif strcmp(Keyword,'PHYS_DATE')
         PHYS_DIR=Value;     
%FS_PULS      
      elseif strcmp(Keyword,'FS_PULS')
         FS_PULS=Value;      

%FS_RESP      
      elseif strcmp(Keyword,'FS_RESP')
         FS_RESP=Value;      
      
%FS_ECG      
      elseif strcmp(Keyword,'FS_ECG')
         FS_ECG=Value;      
         
%RVT
      elseif strcmp(Keyword,'RVT')
         RVT=Value;  
         
%RESPXPULSE_ORDER      
      elseif strcmp(Keyword,'RESPXPULSE_ORDER')
         RESPXPULSE_ORDER=Value;  
         
%RESPXECG_ORDER      
      elseif strcmp(Keyword,'RESPXECG_ORDER')
         RESPXECG_ORDER=Value;     

%FIND_PULSEPEAKS      
      elseif strcmp(Keyword,'FIND_PULSEPEAKS')
         FIND_PULSEPEAKS=Value;   

%FIND_ECGPEAKS      
      elseif strcmp(Keyword,'FIND_ECGPEAKS')
         FIND_ECGPEAKS=Value;   
               
%FIND_RESPPEAKS      
      elseif strcmp(Keyword,'FIND_RESPPEAKS')
         FIND_RESPPEAKS=Value;   
         
%PULSEPEAKS_INTERVAL      
      elseif strcmp(Keyword,'PULSEPEAKS_INTERVAL')
         PULSEPEAKS_INTERVAL=Value;   

%ECGPEAKS_INTERVAL      
      elseif strcmp(Keyword,'ECGPEAKS_INTERVAL')
         ECGPEAKS_INTERVAL=Value;   
               
%RESPPEAKS_INTERVAL      
      elseif strcmp(Keyword,'RESPPEAKS_INTERVAL')
         RESPPEAKS_INTERVAL=Value;   
         
%LP_PULSE      
      elseif strcmp(Keyword,'LP_PULSE')
         LPFILTER_PULSE=Value;   

%LP_PULSE_FSTOP    
      elseif strcmp(Keyword,'LP_PULSE_FSTOP')
         LP_PULSE_FSTOP=Value;   
         
%LP_RESP 
      elseif strcmp(Keyword,'LP_RESP')
         LPFILTER_RESP=Value;   

%LP_RESP_FSTOP    
      elseif strcmp(Keyword,'LP_RESP_FSTOP')
         LP_RESP_FSTOP=Value;   
         
%LP_ECG      
      elseif strcmp(Keyword,'LP_ECG')
         LPFILTER_PULSE=Value;   

%LP_ECG_FSTOP    
      elseif strcmp(Keyword,'LP_ECG_FSTOP')
         LP_ECG_FSTOP=Value;   
         
%ELSE
      else
         error(['Unknown flag: ' Keyword])
      
      end
   end

   
%Get the filenames. Valid options are a filename or a dicom file and a directory with recordings
if ~isempty(PHYS_FILE)
    % choose ON for physfiles if they are in the directory, otherwise OFF
    [PHYS_DIR,PHYS_NAME] = fileparts(PHYS_FILE);
    direc=fullfile(PHYS_DIR,PHYS_NAME);
    e=exist([direc '.resp']);
    if e ~= 0
        RESP='ON';
        display('RESP is ON')
    else
        RESP='OFF';
        display('RESP is OFF')
    end
    e=exist([direc '.puls']);
    if e ~= 0
        PULSE='ON';
        display('PULSE is ON')
    else
        PULSE='OFF';
        display('PULSE is OFF')
    end
    e=exist([direc '.ecg']);
    if e ~= 0
        ECG='ON';
        display('ECG is ON')
    else
        ECG='OFF';
        display('ECG is OFF')
    end
    
    % rename files
	P_phys=PHYS_FILE;
	[PHYS_DIR,PHYS_FILE,EXT] = fileparts(PHYS_FILE);
    if strcmp(TRACETYPE,'SIEMENS')
        if ~(strcmp(EXT,'.resp') |strcmp(EXT,'.puls') |strcmp(EXT,'.ecg') )
            error('This does not appear to be a file of TRACETYPE SIEMENS')
        end
        if strcmp(PULSE,'ON')
            PULSE_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.puls']);
        end
        if strcmp(RESP,'ON')
            RESP_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.resp']);
        end
        if strcmp(ECG,'ON')
            ECG_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.ecg']);
        end
        
    elseif strcmp(TRACETYPE,'IOP')
        if ~(strcmp(EXT,'.rtrig') |strcmp(EXT,'.ptrig') |strcmp(EXT,'.rpts') )
            error('This does not appear to file of TRACETYPE IOP')
        end
        
        if strcmp(PULSE,'ON')
            PULSE_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.rpts']);
            PULSE_ONSETS_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.ptrig']);
        end
        if strcmp(RESP,'ON')
            RESP_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.rpts']);
            RESP_ONSETS_FILE=fullfile(PHYS_DIR,[PHYS_FILE '.rtrig']);
        end
        if strcmp(ECG,'ON')
            error('ECG is not supported for this TRACETYPE')
        end
        
    elseif strcmp(TRACETYPE,'GE')
        i=findstr(PHYS_FILE,'_');
        if ~(strcmp(PHYS_FILE(1:i-1),'RespData')| strcmp(PHYS_FILE(1:i-1),'TrigResp')|strcmp(PHYS_FILE(1:i-1),'ECGData')|strcmp(PHYS_FILE(1:i-1),'TrigEcg') )
            error('This does not appear to file of TRACETYPE GE')
        end
        if strcmp(RESP,'ON')
            RESP_FILE=fullfile(PHYS_DIR,['RespData' PHYS_FILE(i:end)]);
            RESP_ONSETS_FILE=fullfile(PHYS_DIR,['TrigResp' PHYS_FILE(i:end)]);
        end
        if strcmp(ECG,'ON')
            ECG_FILE=fullfile(PHYS_DIR,['ECGData' PHYS_FILE(i:end)]);
            ECG_ONSETS_FILE=fullfile(PHYS_DIR,['TrigEcg' PHYS_FILE(i:end)]);
        end
        if strcmp(PULSE,'ON')
            PULSE_FILE=fullfile(PHYS_DIR,['ECGData' PHYS_FILE(i:end)]);
            PULSE_ONSETS_FILE=fullfile(PHYS_DIR,['TrigEcg' PHYS_FILE(i:end)]);
        end
    end
elseif xor(~isempty(DICOM_FILE),~isempty(DICOM_HDR))
    if (~isempty(DICOM_FILE) & ~isempty(PHYS_DIR)& ~isempty(PHYS_FILTER) & ~isempty(PHYS_DATE) & isempty(DICOM_HDR))
        if license('test','image_toolbox')
            DICOM_HDR=dicominfo(DICOM_FILE);
        elseif exist('spm_dicom_headers')
            DICOM_HDR=spm_dicom_headers(DICOM_FILE);
        else
            error('NO dicom reader found, please install one or use the DICOM_HDR option instead')
        end
        %get the physfile:
    elseif (~isempty(DICOM_HDR) & ~isempty(PHYS_DIR)& ~isempty(PHYS_FILTER) & ~isempty(PHYS_DATE) & isempty(DICOM_FILE))
        error('This feature is not yet implemented ... comming soon')
        %get the physfile
    end
elseif (~isempty(DICOM_FILE) & ~isempty(DICOM_HDR))
    error('Only one of the options DICOM_FILE and DICOM_HDR can be used')    
else
    error('You forgot to provide a valid PHYS_FILE or (DICOM_IMAGE,PHYS_DIR,PHYS_FILTER,PHYS_DATE) or (DICOM_IMAGE,PHYS_DIR,PHYS_FILTER,PHYS_DATE) combination')
end

if strcmp(TRACETYPE,'SIEMENS')
    %Load the pulse loggings

    if strcmp(PULSE,'ON')
        PULSE_RAW=textread(PULSE_FILE,'%s','delimiter','\n','whitespace','','bufsize',10000000);
        START=find(strcmp(PULSE_RAW,START_STAMP));
        STOP=find(strcmp(PULSE_RAW,STOP_STAMP));
     if isempty(START)
         PULSEWARNING=[PULSEWARNING 'No ' START_STAMP 'in .puls file'];
     end

     if isempty(STOP)
         PULSEWARNING=[PULSEWARNING ' No ' STOP_STAMP ' in .puls file. ']
         for j=STOP:length(PULSE_RAW)
             k=findstr(PULSE_RAW{j},'PULSE Freq Per: ');
             if ~isempty(k)
                 l=sscanf(PULSE_RAW{j}(k+15:end),'%f');
                 if l==[0 0]'
                     PULSEWARNING=['This file is incomplete. ' PULSEWARNING]
                 end
             end
         end
         if (isempty(FS_PULSE) & isempty(findstr('PULSEWARNING','This file is incomplete. ')))
            PULSEWARNING=[PULSEWARNING 'Please provide sampling rate and I will try to procede. ']
         end
     end


     if isempty(findstr('PULSEWARNING','This file is incomplete. ')) 
        PULSE_RAW=sscanf(PULSE_RAW{START+1},'%f');
        PULSE_RAW=PULSE_RAW(2:end-1); %Remove first value in the file (6002) and the last value (5002)
        PULSE_TS=PULSE_RAW(PULSE_RAW<5000);% create time_course without inserted peaks
        idx=find(PULSE_RAW==5000); %find siemens detected peaks
        PULSE_PEAKS=PULSE_RAW;
        idx(idx==1)=2;
        PULSE_PEAKS(idx-1)=5000; %substitute signalvalue with peakvalue at the timepoint before the peak
        PULSE_PEAKS(idx)=[]; %erase old detected peaks

        if isempty(findstr(' in .resp file','PULSEWARNING')) 
            display(['TR: ' num2str(TR)])
            display(['NUM_VOLUMES: ' num2str(NUM_VOLUMES)])
            display(['DUMMY_VOLUMES: ' num2str(DUMMY_VOLUMES)])
            FS_PULSE=length(PULSE_TS)/(TR*(NUM_VOLUMES+DUMMY_VOLUMES)); %find samplingrate
        else
            if ~isempty(FS_PULSE)
                display(['Your FS_PULSE of: ' num2str(FS_PULSE) 'Hz will be used'])
            else
                PULSE='OFF';
            end
        end
        if ~strcmp(PULSE,'OFF')
               PULSE_ONSETS=find(PULSE_PEAKS==5000)/FS_PULSE; %find onsets in seconds
  
        end
     end
    else
        PULSE_ONSETS=[];
    end %(PULSE ON)

    %Load the respiration loggings
    if strcmp(RESP,'ON') 
        RESP_RAW=textread(RESP_FILE,'%s','delimiter','\n','whitespace','','bufsize',10000000);
        START=find(strcmp(RESP_RAW,START_STAMP));
        STOP=find(strcmp(RESP_RAW,STOP_STAMP));
     if isempty(START)
         RESPWARNING=[RESPWARNING 'No ' START_STAMP 'in .resp file'];
     end

     if isempty(STOP)
         RESPWARNING=[RESPWARNING ' No ' STOP_STAMP ' in .resp file. ']
         for j=STOP:length(RESP_RAW)
             k=findstr(RESP_RAW{j},'RESP Freq Per: ');
             if ~isempty(k)
                 l=sscanf(RESP_RAW{j}(k+15:end),'%f');
                 if l==[0 0]'
                     RESPWARNING=['This file is incomplete. ' RESPWARNING]
                 end
             end
         end
         if (isempty(FS_RESP) & isempty(findstr('RESPWARNING','This file is incomplete. ')))
            RESPWARNING=[RESPWARNING 'Please provide sampling rate and I will try to procede. ']
         end
     end


     if isempty(findstr('RESPWARNING','This file is incomplete. ')) %%!! added ''
        RESP_RAW=sscanf(RESP_RAW{START+1},'%f');
        RESP_RAW=RESP_RAW(2:end-1); %Remove first value in the file (6002) and the last value (5002)
        RESP_TS=RESP_RAW(RESP_RAW<5000);% create time_course without inserted peaks
        idx=find(RESP_RAW==5000); %find siemens detected peaks
        RESP_PEAKS=RESP_RAW;
        idx(idx==1)=2;
        RESP_PEAKS(idx-1)=5000; %substitute signalvalue with peakvalue at the timepoint before the peak
        RESP_PEAKS(idx)=[]; %erase old detected peaks

        if isempty(findstr(' in .resp file','RESPWARNING'))  %%!! added ''
            FS_RESP=length(RESP_TS)/(TR*(NUM_VOLUMES+DUMMY_VOLUMES)); %find samplingrate
        else
            if ~isempty(FS_RESP)
                display(['Your FS_RESP of: ' num2str(FS_RESP) 'Hz will be used'])
            else
                RESP='OFF';
            end
        end
        if ~strcmp(RESP,'OFF')
            RESP_ONSETS=find(RESP_PEAKS==5000)/FS_RESP; %find onsets in seconds
        end
     end
    else
        RESP_ONSETS=[];
    end %(RESP ON)

        %Load the ECG loggings
    if strcmp(ECG,'ON') 
        ECG_RAW=textread(ECG_FILE,'%s','delimiter','\n','whitespace','','bufsize',10000000);
        START=find(strcmp(ECG_RAW,START_STAMP));
        STOP=find(strcmp(ECG_RAW,STOP_STAMP));
     if isempty(START)
         ECGWARNING=[ECGWARNING 'No ' START_STAMP 'in .ecg file'];
     end

     if isempty(STOP)
         ECGWARNING=[ECGWARNING ' No ' STOP_STAMP ' in .ecg file. ']
         for j=STOP:length(ECG_RAW)
             k=findstr(ECG_RAW{j},'ECG Freq Per: ');
             if ~isempty(k)
                 l=sscanf(ECG_RAW{j}(k+15:end),'%f');
                 if l==[0 0]'
                     ECGWARNING=['This file is incomplete. ' ECGWARNING]
                 end
             end
         end
         if (isempty(FS_ECG) & isempty(findstr('ECGWARNING','This file is incomplete. ')))
            ECGWARNING=[ECGWARNING 'Please provide sampling rate and I will try to procede. ']
         end
     end

     if isempty(findstr('ECGWARNING','This file is incomplete. '))
        ECG_RAW=sscanf(ECG_RAW{START+1},'%f');
        ECG_RAW=ECG_RAW(2:end-1); %Remove first value in the file (6002) and the last value (5002)
        idx=find(ECG_RAW==6000); %find siemens detected extra peaks (unique to the ECG recording)
        ECG_RAW(idx)=[];%erase extra peaks
        ECG_TS=ECG_RAW(ECG_RAW<5000);% create time_course without inserted peaks
        idx=find(ECG_RAW==5000); %find siemens detected peaks
        ECG_PEAKS=ECG_RAW;
        idx(idx==1)=2;
        ECG_PEAKS(idx-1)=5000; %substitute signalvalue with peakvalue at the timepoint before the peak
        ECG_PEAKS(idx)=[]; %erase old detected peaks

        if isempty(findstr(' in .resp file','ECGWARNING'))
            FS_ECG=length(ECG_TS)/(TR*(NUM_VOLUMES+DUMMY_VOLUMES)); %find samplingrate
        else
            if ~isempty(FS_ECG)
                display(['Your FS_ECG of: ' num2str(FS_ECG) 'Hz will be used'])
            else
                ECG='OFF';
            end
        end
        if ~strcmp(ECG,'OFF')
            ECG_ONSETS=find(ECG_PEAKS==5000)/FS_ECG; %find onsets in seconds
        end
     end
    else
        ECG_ONSETS=[];
    end %(ECG ON)

elseif strcmp(TRACETYPE,'GE')
    if strcmp(PULSE,'ON')
        PULSE_TS=load(PULSE_FILE);
        FS_PULSE=length(PULSE_TS)/(NUM_VOLUMES+DUMMY_VOLUMES)/TR;
        PULSE_ONSETS=load(PULSE_ONSETS_FILE)/FS_PULSE;    
    end
    if strcmp(RESP,'ON')
        RESP_TS=load(RESP_FILE);
        FS_RESP=length(RESP_TS)/(NUM_VOLUMES+DUMMY_VOLUMES)/TR;
        RESP_ONSETS=load(RESP_ONSETS_FILE)/FS_RESP;
    end
    if strcmp(ECG,'ON')
        ECG_TS=load(ECG_FILE);
        FS_ECG=length(ECG_TS)/(NUM_VOLUMES+DUMMY_VOLUMES)/TR;
        ECG_ONSETS=load(ECG_ONSETS_FILE)/FS_ECG;
    end
elseif strcmp(TRACETYPE,'IOP')
    [TIME JUNK RESP_TS PULSE_TS]=textread(PULSE_FILE,'','delimiter','\t','headerlines',7);
    if strcmp(PULSE,'ON')
        PULSE_TS=PULSE_TS(1:FS_PULSE*TR*(NUM_VOLUMES+DUMMY_VOLUMES)); %truncate the file to macth the total number of volumes
        [PULSE_ONSETS IBI]=textread(PULSE_ONSETS_FILE,'','delimiter',',','headerlines',1);
        PULSE_ONSETS=PULSE_ONSETS(PULSE_ONSETS<TR*(NUM_VOLUMES+DUMMY_VOLUMES));%truncate the file to macth the total number of volumes
    end
    if strcmp(RESP,'ON')
        RESP_TS=RESP_TS(1:FS_RESP*TR*(NUM_VOLUMES+DUMMY_VOLUMES)); %truncate the file to macth the total number of volumes
        [RESP_ONSETS IBI]=textread(RESP_ONSETS_FILE,'','delimiter',',','headerlines',1);
        RESP_ONSETS=RESP_ONSETS-0.2;% Move the IOP trigger 200ms back in time 
        RESP_ONSETS=RESP_ONSETS(RESP_ONSETS<TR*(NUM_VOLUMES+DUMMY_VOLUMES));%truncate the file to macth the total number of volumes
    end
    if strcmp(ECG,'ON')
        display('ECG not suported for this TRACETYPE')
    end

elseif strcmp(TRACETYPE,'CUSTOM')
    if (isempty(RESP_TS) | isempty(RESP_ONSETS))
        display('For TRACETYPE CUSTOM a set of RESP_ONSET_FILE and RESP_TS must be provided') 
    else
        display('NOT ready yet')
    end

end %(TRACETYPE SIEMENS)

if strcmp(RESP,'ESTIMATE')
    if strcmp(PULSE,'ON')
        display('The respiration will be estimated from the peak to peak variation in the pulse (NOT implemented yet)')
    elseif strcmp(ECG,'ON')
        display('The respiration will be estimated from the peak to peak variation in the ECG  (NOT implemented yet)')
    end
end

% This section uses MATLAB's findpeaks to determine peak locations
if strcmp(FIND_PULSEPEAKS,'ON')
    if strcmp(PULSE,'ON')
        if strcmp(LP_PULSE,'ON')
            FN=FS_PULSE/2;
            [B,A]=cheby2(CHEBY_ORDER,CHEBY_RIP,LP_PULSE_FSTOP/FN,'low');
            PULSE_TS = filtfilt(B,A,PULSE_TS);
        end
        interval=round(FS_PULSE*PULSEPEAKS_INTERVAL);
        [PEAKS PULSE_ONSETS]=findpeaks(PULSE_TS,'MINPEAKDISTANCE',interval);
        PULSE_ONSETS=PULSE_ONSETS/FS_PULSE;
    else
        error('PULSE must be set to ON')
    end
end

if strcmp(FIND_ECGPEAKS,'ON')
    if strcmp(ECG,'ON')
        if strcmp(LP_ECG,'ON')
            FN=FS_ECG/2;
            [B,A]=cheby2(CHEBY_ORDER,CHEBY_RIP,LP_ECG_FSTOP/FN,'low');
            ECG_TS = filtfilt(B,A,ECG_TS);
        end
        interval=round(FS_ECG*ECGPEAKS_INTERVAL);
        [PEAKS ECG_ONSETS]=findpeaks(ECG_TS,'MINPEAKDISTANCE',interval);
        ECG_ONSETS=ECG_ONSETS/FS_ECG;
    else
        error('ECG must be set to ON')
    end
end


if strcmp(FIND_RESPPEAKS,'ON')
    if strcmp(RESP,'ON')
        if strcmp(LP_RESP,'ON')
            FN=FS_RESP/2;
            [B,A]=cheby2(CHEBY_ORDER,CHEBY_RIP,LP_RESP_FSTOP/FN,'low');
            RESP_TS = filtfilt(B,A,RESP_TS);
        end
        interval=round(FS_RESP*RESPPEAKS_INTERVAL);
        [PEAKS RESP_ONSETS]=findpeaks(RESP_TS,'MINPEAKDISTANCE',interval);
        RESP_ONSETS=RESP_ONSETS/FS_RESP;
        RESP_ONSETS=RESP_ONSETS-0.2;% Move the IOP trigger 200ms back in time 
    else
        error('RESP must be set to ON')
    end
end


% Here comes the standard RETROICOR regressors 
% Inspired by Josephs et al 1997 and Glover et al. 2000
% This version does NOT use a special method for estimation the respiratory
% phase. This is the approach used in Lund et al. 2006

for ref=1:length(REF_SLICE)
    % Find the onsets for the reference slice. Note: NUM_VOLUMES is without DUMMY_VOLUMES
    FIRST_ONS=TR*(DUMMY_VOLUMES+((REF_SLICE(ref)-1)/NUM_SLICES));
    
    if REF_SLICE(ref)==1
        REF_SLICE_ONS=FIRST_ONS:TR:(NUM_VOLUMES+DUMMY_VOLUMES-1)*TR;
    elseif REF_SLICE(ref)>NUM_SLICES
            error('REF_SLICE must be a smaller than NUM_SLICES')
    elseif REF_SLICE(ref)>1
        REF_SLICE_ONS=FIRST_ONS:TR:(NUM_VOLUMES+DUMMY_VOLUMES)*TR;
    else
        error('REF_SLICE must be a positive integer')
    end
    
    if strcmp(PULSE,'ON')
        PULSE_PHASE=zeros(length(REF_SLICE_ONS),1);
        PULSE_RETROICOR=zeros(length(REF_SLICE_ONS),PULSE_ORDER*2);
        for i=1:length(REF_SLICE_ONS)
            [ttp idx2]=min(abs(REF_SLICE_ONS(i)-PULSE_ONSETS));
            if REF_SLICE_ONS(i)<PULSE_ONSETS(idx2)
                if idx2==1
                    ttp=REF_SLICE_ONS(i)-(PULSE_ONSETS(idx2)- ... 
                        (PULSE_ONSETS(idx2)+PULSE_ONSETS(idx2+1)));
                    idx2=idx2+1;
                end
                ttp=REF_SLICE_ONS(i)-PULSE_ONSETS(idx2-1);
                idx2=idx2-1;
            end
            if idx2==length(PULSE_ONSETS)
                idx2=idx2-1;
            end
            PULSE_PHASE(i)=2*pi*ttp/(PULSE_ONSETS(idx2+1)-PULSE_ONSETS(idx2));
            for j=1:PULSE_ORDER
                PULSE_RETROICOR(i,2*j-1)=sin(PULSE_PHASE(i)*j);
                PULSE_RETROICOR(i,2*j)=cos(PULSE_PHASE(i)*j);
            end
        end  
    else
        PULSE_RETROICOR=[];
    end

    if strcmp(RESP,'ON')
        RESP_PHASE=zeros(length(REF_SLICE_ONS),1);
        RESP_RETROICOR=zeros(length(REF_SLICE_ONS),RESP_ORDER*2);
        for i=1:length(REF_SLICE_ONS)
            [ttp idx2]=min(abs(REF_SLICE_ONS(i)-RESP_ONSETS));
            if REF_SLICE_ONS(i)<RESP_ONSETS(idx2)
                if idx2==1
                    ttp=REF_SLICE_ONS(i)-(RESP_ONSETS(idx2)- ... 
                        (RESP_ONSETS(idx2)+RESP_ONSETS(idx2+1)));
                    idx2=idx2+1;
                end
                ttp=REF_SLICE_ONS(i)-RESP_ONSETS(idx2-1);
                idx2=idx2-1;
            end
            if idx2==length(RESP_ONSETS)
                idx2=idx2-1;
            end
            RESP_PHASE(i)=2*pi*ttp/(RESP_ONSETS(idx2+1)-RESP_ONSETS(idx2));
            for j=1:RESP_ORDER
                RESP_RETROICOR(i,2*j-1)=sin(RESP_PHASE(i)*j);
                RESP_RETROICOR(i,2*j)=cos(RESP_PHASE(i)*j);
            end
        end  
    else
        RESP_RETROICOR=[];
    end


    if strcmp(ECG,'ON')
        ECG_PHASE=zeros(length(REF_SLICE_ONS),1);
        ECG_RETROICOR=zeros(length(REF_SLICE_ONS),ECG_ORDER*2);
        for i=1:length(REF_SLICE_ONS)
            [ttp idx2]=min(abs(REF_SLICE_ONS(i)-ECG_ONSETS));
            if REF_SLICE_ONS(i)<ECG_ONSETS(idx2)
                if idx2==1
                    ttp=REF_SLICE_ONS(i)-(ECG_ONSETS(idx2)- ... 
                        (ECG_ONSETS(idx2)+ECG_ONSETS(idx2+1)));
                    idx2=idx2+1;
                end
                ttp=REF_SLICE_ONS(i)-ECG_ONSETS(idx2-1);
                idx2=idx2-1;
            end
            if idx2==length(ECG_ONSETS)
                idx2=idx2-1;
            end
            ECG_PHASE(i)=2*pi*ttp/(ECG_ONSETS(idx2+1)-ECG_ONSETS(idx2));
            for j=1:ECG_ORDER
                ECG_RETROICOR(i,2*j-1)=sin(ECG_PHASE(i)*j);
                ECG_RETROICOR(i,2*j)=cos(ECG_PHASE(i)*j);
            end
        end 
    else
        ECG_RETROICOR=[];
    end



    % The RVT regressor was suggested by Birn et al. 2006 and is based on work by Wise et al 2004 
    if strcmp(RVT,'ON')
        if ~(strcmp(RESP,'ON')|strcmp(RESP,'ESTIMATE'))
            RVTWARNING='RVT regressor cannot be constructed RESP has to be ON or ESTIMATE';
            display(RVTWARNING)
        elseif strcmp(RVT,'ESTIMATE')
            RVTWARNING='RVT regressor cannot be constructed RESP has to be ON or ESTIMATE';
            display(RVTWARNING)
        else
            %only run redundant for first reference slice to reduce
            %computation time
            if ref==1
                % First we make a vector containing the breath to breath period and
                % frequency:
                BREATH_TO_BREATH_PERIOD=diff(RESP_ONSETS);
                BREATH_TO_BREATH_FREQUENCY=1./BREATH_TO_BREATH_PERIOD;

                % These frequencies are then associated with the timepoint between the
                % onsets:
                BREATH_TO_BREATH_FREQUENCY_ONSETS=RESP_ONSETS(1:end-1)+BREATH_TO_BREATH_PERIOD/2;

                TIME=(1:round(length(RESP_TS)))./FS_RESP;

                MINAMPTIME=zeros(1,length(RESP_ONSETS));
                MAXAMPTIME=zeros(1,length(RESP_ONSETS));
                MINAMP=zeros(1,length(RESP_ONSETS));
                MAXAMP=zeros(1,length(RESP_ONSETS));

                [MAXAMP(1) AMPIDX]=max(RESP_TS(1:round(min(find(TIME>=RESP_ONSETS(1))))));
                MAXAMPTIME(1)=TIME(1)+((AMPIDX-1)/FS_RESP);

                    for j=2:length(RESP_ONSETS)
                        SIDX=find(TIME>=RESP_ONSETS(j-1) & TIME<=RESP_ONSETS(j));
                        [MAXAMP(j) AMPIDX]=max(RESP_TS(SIDX(2:end)));
                        MAXAMPTIME(j)=TIME(SIDX(2))+((AMPIDX-1)/FS_RESP);
                    end

                    for j=1:length(MAXAMPTIME)               
                        if j==length(MAXAMPTIME)
                            SIDX=find(TIME>=MAXAMPTIME(j));
                        else
                            SIDX=find(TIME>=MAXAMPTIME(j) & TIME<=MAXAMPTIME(j+1));
                        end
                        [MINAMP(j) AMPIDX]=min(RESP_TS(SIDX(2:end)));
                        MINAMPTIME(j)=TIME(SIDX(2))+((AMPIDX-1)/FS_RESP);
                    end

                    if MAXAMPTIME(1)>3./FS_RESP
                         RESPAMP1=MAXAMP-MINAMP;
                         RESPAMP2=MAXAMP(2:end)-MINAMP(1:end-1);
                         MINAMPTIME=MINAMPTIME(:);
                         MAXAMPTIME=MAXAMPTIME(:);
                         RESPAMPTIME1=mean([MINAMPTIME MAXAMPTIME]');
                         RESPAMPTIME2=mean([MINAMPTIME(1:end-1) MAXAMPTIME(2:end)]');
                    else
                         MAXAMPTIME(1)=[];
                         MAXAMP(1)=[];
                         RESPAMP1=MAXAMP-MINAMP(2:end);
                         RESPAMP2=MAXAMP-MINAMP(1:end-1);
                         MINAMPTIME=MINAMPTIME(:);
                         MAXAMPTIME=MAXAMPTIME(:);
                         RESPAMPTIME1=mean([MINAMPTIME(2:end) MAXAMPTIME]');
                         RESPAMPTIME2=mean([MINAMPTIME(1:end-1) MAXAMPTIME]');
                    end
                    RESPAMPTIME=[RESPAMPTIME1 RESPAMPTIME2];
                    RESPAMP=[RESPAMP1 RESPAMP2];
                    [RESPAMPTIME RESPSORTIDX]=sort(RESPAMPTIME);
                    RESPAMP=RESPAMP(RESPSORTIDX);
                    RESPAMP=RESPAMP(:);
            end

                RESPAMP_TR=interp1(RESPAMPTIME,RESPAMP,REF_SLICE_ONS,'nearest','extrap');
                RESPAMP_TR=RESPAMP_TR(:);

                RESP_PERIOD_TR=interp1(BREATH_TO_BREATH_FREQUENCY_ONSETS,1./BREATH_TO_BREATH_FREQUENCY,REF_SLICE_ONS,'nearest','extrap');
                RESP_PERIOD_TR=RESP_PERIOD_TR(:);

                RVT_RAW=RESPAMP_TR./RESP_PERIOD_TR-mean(RESPAMP_TR./RESP_PERIOD_TR);



                if ~isempty(RVT_DELAY)
                    for i=1:length(RVT_DELAY)
                        RESPAMP_TR_SHIFT=interp1(RESPAMPTIME,RESPAMP,REF_SLICE_ONS-RVT_DELAY(i),'nearest','extrap');
                        RESP_PERIOD_TR_SHIFT=interp1(BREATH_TO_BREATH_FREQUENCY_ONSETS,1./BREATH_TO_BREATH_FREQUENCY,REF_SLICE_ONS-RVT_DELAY(i),'nearest','extrap');
                        RVT_DELAYED(:,i)=RESPAMP_TR_SHIFT./RESP_PERIOD_TR_SHIFT-mean(RESPAMP_TR_SHIFT./RESP_PERIOD_TR_SHIFT);
                    end
                    if length(RVT_DELAY)==1 % make delayed RVT and differential RVT
                        DIFF_RVT_DELAYED=[0 ;diff(RVT_DELAYED)];
                    end
                end
        end
    else
        RVT_DELAYED=[];
        DIFF_RVT_DELAYED=[];
    end



    % The interaction regressors (between heartbeat and respiration is described
    % Brooks et al. NeuroImage doi: 10.1016/j.neuroimage.2007.09.018

    if strcmp(RESPXPULSE,'ON')
        if ~(strcmp(PULSE,'ON') & strcmp(RESP,'ON'))
            error('Both PULSE and RESP is needed')
        end
        for i=1:RESPXPULSE_ORDER
            RESPXPULSE_RETROICOR(:,4*i-3)=sin((PULSE_PHASE+RESP_PHASE)*i);
            RESPXPULSE_RETROICOR(:,4*i-2)=cos((PULSE_PHASE+RESP_PHASE)*i);
            RESPXPULSE_RETROICOR(:,4*i-1)=sin((PULSE_PHASE-RESP_PHASE)*i);
            RESPXPULSE_RETROICOR(:,4*i)=cos((PULSE_PHASE-RESP_PHASE)*i);
        end
    end

    if strcmp(RESPXECG,'ON')
        if ~(strcmp(ECG,'ON') & strcmp(RESP,'ON'))
            error('Both ECG and RESP is needed')
        end
        for i=1:RESPXECG_ORDER
            RESPXECG_RETROICOR(:,4*i-3)=sin((ECG_PHASE+RESP_PHASE)*i);
            RESPXECG_RETROICOR(:,4*i-2)=cos((ECG_PHASE+RESP_PHASE)*i);
            RESPXECG_RETROICOR(:,4*i-1)=sin((ECG_PHASE-RESP_PHASE)*i);
            RESPXECG_RETROICOR(:,4*i)=cos((ECG_PHASE-RESP_PHASE)*i);
        end
    end
    
    NVRstruct(ref).tracetype=TRACETYPE;
    NVRstruct(ref).dicom_file=DICOM_FILE;
    NVRstruct(ref).phys_file=PHYS_FILE;
    NVRstruct(ref).phys_dir=PHYS_DIR;
    NVRstruct(ref).stop_stamp=STOP_STAMP;
    NVRstruct(ref).start_stamp=START_STAMP;

    NVRstruct(ref).TR=TR;
    NVRstruct(ref).num_slices=NUM_SLICES;
    NVRstruct(ref).num_volumes=NUM_VOLUMES;
    NVRstruct(ref).dummy_volumes=DUMMY_VOLUMES;
    NVRstruct(ref).ref_slice=REF_SLICE(ref);
    NVRstruct(ref).ref_slice_ons=REF_SLICE_ONS;

    NVRstruct(ref).pulse_onsets=PULSE_ONSETS;
    NVRstruct(ref).pulse_retroicor=PULSE_RETROICOR;
    NVRstruct(ref).resp_onsets=RESP_ONSETS;
    NVRstruct(ref).resp_retroicor=RESP_RETROICOR;
    NVRstruct(ref).ecg_onsets=ECG_ONSETS;
    NVRstruct(ref).ecg_retroicor=ECG_RETROICOR;
    NVRstruct(ref).rvt=[RVT_DELAYED DIFF_RVT_DELAYED]; %%%
    
end
end

