function job=mr_cfg_mark_nuisance
%mark regressors nuisance
%created 12/11-2019
%Kristoffer H. Madsen (kristofferm@drcmr.dk)


regexpr         = cfg_entry;
regexpr.tag     = 'regexpr';
regexpr.name    = 'Regular expresion';
regexpr.help    = {'Regular expresion to use for matching nuisance regressors.'};
regexpr.strtype = 's';
regexpr.num     = [1 inf];
regexpr.val     = {'^Sn\(.*\) R[0-9]*$'};


spmmat         = cfg_files;
spmmat.tag     = 'spmmat';
spmmat.name    = 'Select SPM.mat';
spmmat.help    = {
    'Select the SPM.mat file that contains the design specification.'
    'The directory containing this file is known as the input directory.'
    }';
spmmat.filter  = 'mat';
spmmat.ufilter = '^SPM\.mat$';
spmmat.num     = [1 1];




job=cfg_exbranch;
job.tag='MarkNuisance';
job.name='Mark regressors as nuisance';
job.val={spmmat,regexpr};
job.help = {'Mark certain regressors nuisance before estimation via regular expression, this will affect the test for effects of interest mask in which non-sphericity (e.g. temporal autocorrelation) is fitted via REML.'};
job.prog=@run_markn;
job.vout=@vout_markn;


end

function job=run_markn(job)
    load(job.spmmat{1})
    idx = [];
    for j = 1:length(SPM.xX.name)
        if not(isempty(regexp(SPM.xX.name{j},job.regexpr)))
            idx = [idx j];
            fprintf('Marking regressor "%i: %s" as nuisance\n',j,SPM.xX.name{j});
        end
    end
    SPM.xX.iC = setdiff(SPM.xX.iC,idx);
    SPM.xX.iG = idx;
    save(job.spmmat{1},'SPM');
end

function dep = vout_markn(job)

dep(1)            = cfg_dep;
dep(1).sname      = 'SPM.mat File';
dep(1).src_output = substruct('.','spmmat');
dep(1).tgt_spec   = cfg_findspec({{'filter','mat','strtype','e'}});

end